dnl
dnl macros for creating the ElCel Configuration Utility
dnl

dnl AC_CHECK_SIZEOF_WITH_INCL(TYPE, INCLUDES [, CROSS-SIZE])
AC_DEFUN(AC_CHECK_SIZEOF_WITH_INCL,
[changequote(<<, >>)dnl
dnl The name to #define.
define(<<AC_TYPE_NAME>>, translit(sizeof_$1, [a-z *], [A-Z_P]))dnl
dnl The cache variable name.
define(<<AC_CV_NAME>>, translit(ac_cv_sizeof_$1, [ *], [_p]))dnl
changequote([, ])dnl
AC_MSG_CHECKING(size of $1)
AC_CACHE_VAL(AC_CV_NAME,
[AC_TRY_RUN([#include <stdio.h>
$2
main()
{
  FILE *f=fopen("conftestval", "w");
  if (!f) exit(1);
  fprintf(f, "%d\n", sizeof($1));
  exit(0);
}], AC_CV_NAME=`cat conftestval`, AC_CV_NAME=0, ifelse([$3], , , AC_CV_NAME=$3))])dnl
AC_MSG_RESULT($AC_CV_NAME)
AC_DEFINE_UNQUOTED(AC_TYPE_NAME, $AC_CV_NAME)
undefine([AC_TYPE_NAME])dnl
undefine([AC_CV_NAME])dnl
])


AC_DEFUN(AC_CREATEFILE_SETS_MODTIME,
    [
        ET_OS_NAME
        dnl darwin does not set the modificatio  time of a file after creation
	if test $os_name != darwin; then
            AC_DEFINE(CREATEFILE_SETS_MODTIME)
        fi
    ])
	


AC_DEFUN(AC_TYPE_SOCKLEN_T,
    [AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
    [
	    ET_OS_NAME
        dnl HP-UX erroneously defines a socklen_t which is not used 
        dnl by the socket API unless _POSIX_SOURCE_EXTENDED is defined
        dnl which is not the case for a standard OpenTop build
	    if test $os_name = hpux; then
            ac_cv_type_socklen_t=no
        else
            AC_TRY_COMPILE(
            [
#include <sys/types.h>
#include <sys/socket.h>
            ],
            [socklen_t len = 42; int val=0; return getsockopt(0, 0, 0, (void*)&val, &len);],
            ac_cv_type_socklen_t=yes,
            ac_cv_type_socklen_t=no)
            ])
        fi
        if test $ac_cv_type_socklen_t = yes; then
          AC_DEFINE(HAVE_WORKING_SOCKLEN_T)
        fi
    ])

AC_DEFUN(AC_UNSIGNED_WCHAR_T,
    [AC_CACHE_CHECK([for wchar_t_unsigned], ac_cv_wchar_unsigned,
    [
        AC_TRY_RUN(
		[/* volatile prevents gcc2 from optimizing the test away on sparcs.  */
		#if !defined(__STDC__) || __STDC__ != 1
		#define volatile
		#endif
		#include <stddef.h>
		main() {
		  volatile wchar_t c = -1; exit(c < 0);
		}],
		 ac_cv_wchar_unsigned=yes,
		 ac_cv_wchar_unsigned=no,
		 ac_cv_wchar_unsigned=no)
   	])
	if test $ac_cv_wchar_unsigned = yes; then
	  AC_DEFINE(HAVE_UNSIGNED_WCHAR_T)
	fi
    ])

AC_DEFUN(AC_FUNC_THREAD_SAFE_RESOLVER,
    [AC_MSG_CHECKING(for thread-safe resolver functions)
	ET_OS_NAME
	if test $os_name = hpux; then
	  AC_DEFINE(HAVE_THREAD_SAFE_RESOLVER)
	fi
    ])

AC_DEFUN(AC_FUNC_WHICH_GETHOSTBYNAME_R,
    [AC_MSG_CHECKING(for gethostbyname_r)
    AC_LANG_CPLUSPLUS
    if test -z "$ac_cv_gethostbyname_args"; then
        AC_TRY_COMPILE(
	    [
#include <sys/types.h>
#include <netdb.h>],
	    [
struct hostent *hp;
struct hostent h;
char *name;
char buffer[10];
int  h_errno;
hp = gethostbyname_r(name, &h, buffer, 10, &h_errno);],
            ac_cv_gethostbyname_args=5)
    fi
    if test -z "$ac_cv_gethostbyname_args"; then
        AC_TRY_COMPILE(
	    [
#include <sys/types.h>
#include <netdb.h>],
	    [
struct hostent h;
struct hostent_data hdata;
char *name;
int  rc;
rc = gethostbyname_r(name, &h, &hdata);],
            ac_cv_gethostbyname_args=3)
    fi
    if test -z "$ac_cv_gethostbyname_args"; then
        AC_TRY_COMPILE(
	    [
#include <sys/types.h>
#include <netdb.h>],
	    [
struct hostent h;
struct hostent *hp;
char *name;
char buf[10];
int rc;
int h_errno;

rc = gethostbyname_r(name, &h, buf, 10, &hp, &h_errno);
	    ],
	    ac_cv_gethostbyname_args=6)
    fi
    if test -z "$ac_cv_gethostbyname_args"; then
        AC_MSG_RESULT(no)
        have_missing_r_funcs="$have_missing_r_funcs gethostbyname_r"
    else
        if test "$ac_cv_gethostbyname_args" = 3; then
            AC_DEFINE(HAVE_GETHOSTBYNAME_R_3)
        elif test "$ac_cv_gethostbyname_args" = 5; then
            AC_DEFINE(HAVE_GETHOSTBYNAME_R_5)
        elif test "$ac_cv_gethostbyname_args" = 6; then
            AC_DEFINE(HAVE_GETHOSTBYNAME_R_6)
        fi
        AC_MSG_RESULT([yes, and it takes $ac_cv_gethostbyname_args arguments])
    fi
    AC_LANG_RESTORE
])

AC_DEFUN(AC_FUNC_WHICH_GETHOSTBYADDR_R,
    [AC_MSG_CHECKING(for gethostbyaddr_r)
    AC_LANG_CPLUSPLUS
    if test -z "$ac_cv_gethostbyaddr_args"; then
	AC_TRY_COMPILE(
	    [
#include <sys/types.h>
#include <netdb.h>],
	    [
                char * address;
                int length;
                int type;
                struct hostent h;
                struct hostent_data hdata;
                int rc;
                rc = gethostbyaddr_r(address, length, type, &h, &hdata);
	    ],
	    ac_cv_gethostbyaddr_args=5)
	
    fi
    if test -z "$ac_cv_gethostbyaddr_args"; then
	AC_TRY_COMPILE(
	    [
#include <sys/types.h>
#include <netdb.h>],
	    [
                char * address;
                int length;
                int type;
                struct hostent h;
		char buffer[10];
		int buflen;
		int h_errnop;
		struct hostent * hp;

		hp = gethostbyaddr_r(address, length, type, &h,
				     buffer, buflen, &h_errnop);
	    ],
	    ac_cv_gethostbyaddr_args=7)
    fi

    if test -z "$ac_cv_gethostbyaddr_args"; then
	AC_TRY_COMPILE(
	    [
#include <sys/types.h>
#include <netdb.h>],
	    [
                char * address;
                int length;
                int type;
                struct hostent h;
		char buffer[10];
		int buflen;
		int h_errnop;
		struct hostent * hp;
		int rc;

		rc = gethostbyaddr_r(address, length, type, &h,
				     buffer, buflen, &hp, &h_errnop);
	    ],
	    ac_cv_gethostbyaddr_args=8)
    fi
    if test -z "$ac_cv_gethostbyaddr_args"; then
        AC_MSG_RESULT(no)
        AC_DEFINE(MISSING_REENTRANT_RESOLVER)
        have_missing_r_funcs="$have_missing_r_funcs gethostbyaddr_r"
    else
        if test "$ac_cv_gethostbyaddr_args" = 5; then
            AC_DEFINE(HAVE_GETHOSTBYADDR_R_5)
        elif test "$ac_cv_gethostbyaddr_args" = 7; then
            AC_DEFINE(HAVE_GETHOSTBYADDR_R_7)
        elif test "$ac_cv_gethostbyaddr_args" = 8; then
            AC_DEFINE(HAVE_GETHOSTBYADDR_R_8)
        fi
        AC_MSG_RESULT([yes, and it takes $ac_cv_gethostbyaddr_args arguments])
    fi
    AC_LANG_RESTORE
])

AC_DEFUN(ET_OS_NAME,
[
	AC_REQUIRE([AC_CANONICAL_HOST])
	changequote(<<, >>)dnl
	os_name=`echo $host_os | sed 's/^\([a-zA-Z]*\)\(.*\)$/\1/'`
	changequote([, ])dnl
	AC_SUBST(os_name)dnl
])


