dnl ElCel Technology Test Harness
dnl
dnl	Common M4 Macros 
dnl
dnl
divert(-1)

changecom(`/*', `*/')

define(OT_TEST_INIT, `
#`include' "ot/base/System.h"
#`include' "ot/io/Console.h"
#`include' "ot/base/Exception.h"
#`include' "ot/base/StringUtils.h"
#`include' "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);
')

define(OT_TEST_MAIN, `
#`include' "ot/base/System.h"
#`include' "ot/auxil/MemCheckSystemMonitor.h"
#`include' "ot/auxil/CommandLineParser.h"
#`include' "ot/auxil/BasicOption.h"
#`include' "ot/auxil/TraceHelper.h"

using namespace ot;
using namespace ot::auxil;

int fail(0), pass(0);
bool bShowTests(false), bShowPasses(false), bShowFailures(true);
AttributeListParser TestAttributes;

String getTestAttribute(const String& name)
{
	return TestAttributes.getAttributeValue(name);
}

void testMessage(const String& msg)
{
	if (bShowTests) Console::Out()->println(msg);
}

void testFailed(const String& test)
{
	fail++;
	if(bShowFailures) Console::Out()->println(String(OT_T("Test failed for ")) + test);
}

void uncaughtException(const String& e, const String& test)
{
	fail++;
	if(bShowFailures) Console::Out()->println(String(OT_T("Unexpected exception: ")) + e + String(OT_T(" for ")) + test);
}

void testPassed(const String& test)
{
	pass++;
	if(bShowPasses) Console::Out()->println(String(OT_T("Test passed for ")) + test);
}

void goodCatch(const String& test, const String& eMsg)
{
	if(bShowPasses) Console::Out()->println(String(OT_T("Caught expected exception: ")) + eMsg);
	testPassed(test);
}

int main(int argc, char* argv[])
{
	MemCheckSystemMonitor _monitor;

	CommandLineParser cmdlineParser;

	BasicOption optVerbose(OT_T("verbose"), 0x76, BasicOption::none);
	BasicOption optTrace(OT_T("trace"), 0x74, BasicOption::mandatory);
	BasicOption optParam(OT_T("param"), 0x70, BasicOption::mandatory);

	cmdlineParser.addOption(&optVerbose);
	cmdlineParser.addOption(&optTrace);
	cmdlineParser.addOption(&optParam);

	try
	{
		cmdlineParser.parse(argc, argv);
	}
	catch (CommandLineException& e)
	{
		Console::Err()->println(cmdlineParser.getProgramName() + String(OT_T(": ")) + e.getMessage());
		Console::Err()->println(String(OT_T("Try ")) + cmdlineParser.getProgramName() + OT_T(" --help"));
		return (1);
	}

	//
	// Use a TraceHelper to set up a standard tracing output
	//
	if(optTrace.isPresent())
	{
		Tracer::SetTracer(new TraceHelper(cmdlineParser.getProgramName(), optTrace.getArgument()));
	}

	bShowPasses = optVerbose.isPresent();
	bShowTests = optVerbose.isPresent();
	if(optParam.isPresent())
	{
		TestAttributes.parseString(optParam.getArgument());
	}
	
	try
	{
')

define(OT_TEST_EXIT, `

	}
	catch(Exception& e)
	{
		Console::cout() << cmdlineParser.getProgramName() << OT_T(": unhandled exception: ") << e.toString() << endl;
	}
	catch(...)
	{
		Console::cout() << cmdlineParser.getProgramName() << OT_T(": unhandled system exception") << endl;
	}
	Console::Out()->print(cmdlineParser.getProgramName() + OT_T(": tests passed: "));
	Console::Out()->print(pass);
	Console::Out()->print(OT_T(", tests failed: "));
	Console::Out()->println(fail);

	return (fail != 0);
}
')

define(OT_START_TESTS, `
void $1_Tests()
{
	testMessage(OT_T("Starting tests for $1"));')

define(OT_END_TESTS, `
	testMessage(OT_T("End of tests for $1"));
}')

define(OT_PRINT_LINE, `Console::Out()->println(OT_T("$1"))')
define(OT_TEST_DECLARE, `void $1_Tests();')

define(OT_TEST_MESSAGE, `testMessage($1);')
define(OT_TEST_PASSED, `testPassed(OT_T("$1"));')
define(OT_GOOD_CATCH,  `goodCatch(OT_T("$1"), e.toString());')
define(OT_TEST_FAILED, `testFailed(OT_T("$1"));')

define(OT_TEST_BOOL, `OT_TRY_FUNC(if($2) {OT_TEST_PASSED($1)} else {OT_TEST_FAILED($1)}, $1)')
define(OT_TEST_VOID, `OT_TRY_FUNC($2; OT_TEST_PASSED($1), $1)')
define(OT_TEST_THROW, `try
	{
		$2;
		OT_TEST_FAILED($1)
	}
	catch($3& e)
	{
		OT_GOOD_CATCH($1)
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("$1"));
	}')

define(OT_TRY_FUNC, `try
	{
		$1
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("$2"));
	}')

divert
