//
// ElCel Technology Test Harness 
//
// Class: StirngUTils
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/base/Character.h"
#include "ot/base/StringUtils.h"

using namespace ot; 


void StringUtils_Tests()
{
	testMessage(OT_T("Starting tests for StringUtils"));

    //
	String s1 = OT_T("abcdef");
	String s2 = OT_T("ABCDEF");
	const char * pS1 = "abcdef";
	const char * pS2 = "ABCDEF";

	try
	{
		if(StringUtils::CompareNoCase(s1, s2) == 0) {testPassed(OT_T("cmpncase1"));} else {testFailed(OT_T("cmpncase1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("cmpncase1"));
	}
	try
	{
		if(StringUtils::CompareNoCase(pS1, pS2) == 0) {testPassed(OT_T("cmpncase2"));} else {testFailed(OT_T("cmpncase2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("cmpncase2"));
	}
	try
	{
		if(s2 == StringUtils::ToUpper(s1)) {testPassed(OT_T("toUpper"));} else {testFailed(OT_T("toUpper"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("toUpper"));
	}
	try
	{
		if(s1 == StringUtils::ToLower(s2)) {testPassed(OT_T("toLower"));} else {testFailed(OT_T("toLower"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("toLower"));
	}

	Character pound(163UL);
	ByteString strPound = StringUtils::ToLatin1(pound.toString());
	try
	{
		if(strPound[0] == char(163U)) {testPassed(OT_T("ToLatin1"));} else {testFailed(OT_T("ToLatin1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("ToLatin1"));
	};

	const char pLatin1[] = "123.00 please!";
	String sLatin1 = pound.toString() + OT_T("123.00 please!");
	try
	{
		if(sLatin1 == StringUtils::FromLatin1(pLatin1)) {testPassed(OT_T("fromLatin1"));} else {testFailed(OT_T("fromLatin1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("fromLatin1"));
	}
	try
	{
		if(sLatin1 == StringUtils::FromLatin1(pLatin1, sizeof(pLatin1)-1)) {testPassed(OT_T("fromLatin1-2"));} else {testFailed(OT_T("fromLatin1-2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("fromLatin1-2"));
	}

#if !defined(OT_UCS2)
	Character hiChar(0x10f000UL);
#if defined(OT_UCS4)
	bool bMultiExpected = false;
#else
	bool bMultiExpected = true;
#endif
	try
	{
		if(StringUtils::ContainsMultiCharSequence(hiChar.toString()) == bMultiExpected) {testPassed(OT_T("ContainsMultiCharSequence"));} else {testFailed(OT_T("ContainsMultiCharSequence"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("ContainsMultiCharSequence"));
	}
#endif

	String repl1 = OT_T(" ppplease  pppick  uppp  the  ppphone ");
	String repl2 = OT_T(" please  pick  up  the  phone ");
	String repl3 = OT_T("please  pick  up  the  phone");
	String repl4 = OT_T("please pick up the phone");

	String repl = repl1;
	try
	{
		StringUtils::ReplaceAll(repl, OT_T("ppp"), OT_T("p")); testPassed(OT_T("ReplaceAll"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("ReplaceAll"));
	}
	try
	{
		if(repl == repl2) {testPassed(OT_T("ReplaceAll"));} else {testFailed(OT_T("ReplaceAll"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("ReplaceAll"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(repl2, StringUtils::both) == repl3) {testPassed(OT_T("StripWhiteSpace"));} else {testFailed(OT_T("StripWhiteSpace"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(repl3, StringUtils::both) == repl3) {testPassed(OT_T("StripWhiteSpace1"));} else {testFailed(OT_T("StripWhiteSpace1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace1"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(OT_T("   "), StringUtils::both).empty()) {testPassed(OT_T("StripWhiteSpace2"));} else {testFailed(OT_T("StripWhiteSpace2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace2"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(OT_T("i\r\n"), StringUtils::both) == String(OT_T("i"))) {testPassed(OT_T("StripWhiteSpace3"));} else {testFailed(OT_T("StripWhiteSpace3"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace3"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(OT_T("  i\r\n"), StringUtils::both) == String(OT_T("i"))) {testPassed(OT_T("StripWhiteSpace4"));} else {testFailed(OT_T("StripWhiteSpace4"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace4"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(OT_T(" i"), StringUtils::both) == String(OT_T("i"))) {testPassed(OT_T("StripWhiteSpace5"));} else {testFailed(OT_T("StripWhiteSpace5"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace5"));
	}
	try
	{
		if(StringUtils::StripWhiteSpace(OT_T("i"), StringUtils::both) == String(OT_T("i"))) {testPassed(OT_T("StripWhiteSpace6"));} else {testFailed(OT_T("StripWhiteSpace6"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("StripWhiteSpace6"));
	}
	try
	{
		if(StringUtils::NormalizeWhiteSpace(repl3) == repl4) {testPassed(OT_T("NormalizeWS"));} else {testFailed(OT_T("NormalizeWS"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("NormalizeWS"));
	}
	try
	{
		if(StringUtils::IsHexString("xyz") == false) {testPassed(OT_T("IsHexString1"));} else {testFailed(OT_T("IsHexString1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("IsHexString1"));
	}
	try
	{
		if(StringUtils::IsHexString("fab") == true) {testPassed(OT_T("IsHexString1"));} else {testFailed(OT_T("IsHexString1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("IsHexString1"));
	}


	testMessage(OT_T("End of tests for StringUtils"));
}

