//
// ElCel Technology Test Harness
//
// Class: BufferedInputStream
//





#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/io/BufferedInputStream.h"
#include "ot/io/ByteArrayInputStream.h"
#include "ot/base/NullPointerException.h"
#include "ot/base/IllegalArgumentException.h"
#include "ot/io/IOException.h"

#include <memory.h>

using namespace ot::io;


void BufferedInputStream_Tests()
{
	testMessage(OT_T("Starting tests for BufferedInputStream"));

	//
	// Basic API tests
	//

	try
	{
		RefPtr<BufferedInputStream> rpBufferedInputStream =
	              new BufferedInputStream(NULL);
		testFailed(OT_T("constructor 1 null ptr exception"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("constructor 1 null ptr exception"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("constructor 1 null ptr exception"));
	}

	try
	{
		RefPtr<BufferedInputStream> rpBufferedInputStream =
	              new BufferedInputStream(NULL, 1);
		testFailed(OT_T("constructor 2 null ptr exception"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("constructor 2 null ptr exception"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("constructor 2 null ptr exception"));
	}

	const char data[] = "Hello World\n";

	RefPtr<ByteArrayInputStream> rpIS1 = new ByteArrayInputStream((const Byte*)data, sizeof(data));
	RefPtr<BufferedInputStream> rpBIS = new BufferedInputStream(rpIS1.get());
	try
	{
		if(rpBIS->available() == sizeof(data)) {testPassed(OT_T("available"));} else {testFailed(OT_T("available"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("available"));
	}
	try
	{
		if(rpBIS->markSupported()) {testPassed(OT_T("markSupported"));} else {testFailed(OT_T("markSupported"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("markSupported"));
	}

	// Allocate a small byte buffer to perform read operations
	const int iBufLen = 4;
	Byte buffer[iBufLen];
	long l,j;

	try
	{
		rpBIS->read(NULL,1);
		testFailed(OT_T("read throws NullPointerException"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("read throws NullPointerException"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read throws NullPointerException"));
	}
	try
	{
		rpBIS->read(buffer, 0);
		testFailed(OT_T("read throws IllegalArgumentException"));
	}
	catch(IllegalArgumentException& e)
	{
		goodCatch(OT_T("read throws IllegalArgumentException"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read throws IllegalArgumentException"));
	}

	try
	{
		if((l=rpBIS->read(buffer, iBufLen))==iBufLen) {testPassed(OT_T("bytes read 1"));} else {testFailed(OT_T("bytes read 1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("bytes read 1"));
	}
	try
	{
		if(::memcmp(buffer, data, l) == 0) {testPassed(OT_T("data read ok 1"));} else {testFailed(OT_T("data read ok 1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("data read ok 1"));
	}

	try
	{
		if((j=rpBIS->read(buffer, iBufLen))==iBufLen) {testPassed(OT_T("bytes read 2"));} else {testFailed(OT_T("bytes read 2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("bytes read 2"));
	}
	try
	{
		if(::memcmp(buffer, data+l, j) == 0) {testPassed(OT_T("data read ok 2"));} else {testFailed(OT_T("data read ok 2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("data read ok 2"));
	}

	try
	{
		if(rpBIS->read()==*(data+l+j)) {testPassed(OT_T("byte read"));} else {testFailed(OT_T("byte read"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("byte read"));
	}


	testMessage(OT_T("End of tests for BufferedInputStream"));
}

