//
// ElCel Technology Test Harness
//
// Class: BufferedInputStream
//

include(../common.m4)

OT_TEST_INIT

#`include' "ot/io/BufferedInputStream.h"
#`include' "ot/io/ByteArrayInputStream.h"
#`include' "ot/base/NullPointerException.h"
#`include' "ot/base/IllegalArgumentException.h"
#`include' "ot/io/IOException.h"

#`include' <memory.h>

using namespace ot::io;

OT_START_TESTS(`BufferedInputStream')

	//
	// Basic API tests
	//

	OT_TEST_THROW(constructor 1 null ptr exception,
	              RefPtr<BufferedInputStream> rpBufferedInputStream =
	              new BufferedInputStream(NULL), NullPointerException)

	OT_TEST_THROW(constructor 2 null ptr exception,
	              RefPtr<BufferedInputStream> rpBufferedInputStream =
	              new BufferedInputStream(NULL, 1), NullPointerException)

	const char data[] = "Hello World\n";

	RefPtr<ByteArrayInputStream> rpIS1 = new ByteArrayInputStream((const Byte*)data, sizeof(data));
	RefPtr<BufferedInputStream> rpBIS = new BufferedInputStream(rpIS1.get());
	OT_TEST_BOOL(available, rpBIS->available() == sizeof(data))
	OT_TEST_BOOL(markSupported, rpBIS->markSupported())

	// Allocate a small byte buffer to perform read operations
	const int iBufLen = 4;
	Byte buffer[iBufLen];
	long l,j;

	OT_TEST_THROW(read throws NullPointerException, rpBIS->read(NULL,1), NullPointerException)
	OT_TEST_THROW(read throws IllegalArgumentException, rpBIS->read(buffer, 0), IllegalArgumentException)

	OT_TEST_BOOL(bytes read 1, (l=rpBIS->read(buffer, iBufLen))==iBufLen)
	OT_TEST_BOOL(data read ok 1, ::memcmp(buffer, data, l) == 0)

	OT_TEST_BOOL(bytes read 2, (j=rpBIS->read(buffer, iBufLen))==iBufLen)
	OT_TEST_BOOL(data read ok 2, ::memcmp(buffer, data+l, j) == 0)

	OT_TEST_BOOL(byte read, rpBIS->read()==*(data+l+j))

OT_END_TESTS(`BufferedInputStream')

