//
// ElCel Technology Test Harness
//
// Class: BufferedReader
//





#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/io/BufferedReader.h"
#include "ot/base/NullPointerException.h"
#include "ot/base/IllegalArgumentException.h"
#include "ot/io/ByteArrayInputStream.h"
#include "ot/io/InputStreamReader.h"

using namespace ot::io;


void BufferedReader_Tests()
{
	testMessage(OT_T("Starting tests for BufferedReader"));

	//
	// Basic API tests
	//

	try
	{
		RefPtr<BufferedReader> rpBufferedReader =
	              new BufferedReader(NULL);
		testFailed(OT_T("constructor 1 null ptr exception"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("constructor 1 null ptr exception"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("constructor 1 null ptr exception"));
	}

	try
	{
		RefPtr<BufferedReader> rpBufferedReader =
	              new BufferedReader(NULL, 0);
		testFailed(OT_T("constructor 2 null ptr exception"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("constructor 2 null ptr exception"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("constructor 2 null ptr exception"));
	}

	const char data[] = "Hello\rWorld\n\r\n";

	RefPtr<ByteArrayInputStream> rpIS1 = new ByteArrayInputStream((const Byte*)data, sizeof(data)-1);
	RefPtr<InputStreamReader> rpISR = new InputStreamReader(rpIS1.get());
	RefPtr<BufferedReader> rpRdr = new BufferedReader(rpISR.get());
	try
	{
		if(rpRdr->markSupported()) {testPassed(OT_T("markSupported"));} else {testFailed(OT_T("markSupported"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("markSupported"));
	}

	try
	{
		rpRdr->read(NULL,1);
		testFailed(OT_T("read throws NullPointerException"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("read throws NullPointerException"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read throws NullPointerException"));
	}
	CharType buffer[1];
	try
	{
		rpRdr->read(buffer, 0);
		testFailed(OT_T("read throws IllegalArgumentException"));
	}
	catch(IllegalArgumentException& e)
	{
		goodCatch(OT_T("read throws IllegalArgumentException"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read throws IllegalArgumentException"));
	}

	//
	// Test readline() functionality
	//
	String line;
	try
	{
		if((5 == rpRdr->readLine(line) && line == OT_T("Hello"))) {testPassed(OT_T("readline 1"));} else {testFailed(OT_T("readline 1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("readline 1"));
	}
	try
	{
		if((5 == rpRdr->readLine(line) && line == OT_T("World"))) {testPassed(OT_T("readline 2"));} else {testFailed(OT_T("readline 2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("readline 2"));
	}
	try
	{
		if((0 == rpRdr->readLine(line) && line.empty())) {testPassed(OT_T("readline 3"));} else {testFailed(OT_T("readline 3"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("readline 3"));
	}
	try
	{
		if((BufferedReader::EndOfFile == rpRdr->readLine(line) && line.empty())) {testPassed(OT_T("readline 4"));} else {testFailed(OT_T("readline 4"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("readline 4"));
	}


	testMessage(OT_T("End of tests for BufferedReader"));
}

