//
// ElCel Technology Test Harness
//
// Class: BufferedReader
//

include(../common.m4)

OT_TEST_INIT

#`include' "ot/io/BufferedReader.h"
#`include' "ot/base/NullPointerException.h"
#`include' "ot/base/IllegalArgumentException.h"
#`include' "ot/io/ByteArrayInputStream.h"
#`include' "ot/io/InputStreamReader.h"

using namespace ot::io;

OT_START_TESTS(`BufferedReader')

	//
	// Basic API tests
	//

	OT_TEST_THROW(constructor 1 null ptr exception,
	              RefPtr<BufferedReader> rpBufferedReader =
	              new BufferedReader(NULL), NullPointerException)

	OT_TEST_THROW(constructor 2 null ptr exception,
	              RefPtr<BufferedReader> rpBufferedReader =
	              new BufferedReader(NULL, 0), NullPointerException)

	const char data[] = "Hello\rWorld\n\r\n";

	RefPtr<ByteArrayInputStream> rpIS1 = new ByteArrayInputStream((const Byte*)data, sizeof(data)-1);
	RefPtr<InputStreamReader> rpISR = new InputStreamReader(rpIS1.get());
	RefPtr<BufferedReader> rpRdr = new BufferedReader(rpISR.get());
	OT_TEST_BOOL(markSupported, rpRdr->markSupported())

	OT_TEST_THROW(read throws NullPointerException, rpRdr->read(NULL,1), NullPointerException)
	CharType buffer[1];
	OT_TEST_THROW(read throws IllegalArgumentException, rpRdr->read(buffer, 0), IllegalArgumentException)

	//
	// Test readline() functionality
	//
	String line;
	OT_TEST_BOOL(readline 1, (5 == rpRdr->readLine(line) && line == OT_T("Hello")))
	OT_TEST_BOOL(readline 2, (5 == rpRdr->readLine(line) && line == OT_T("World")))
	OT_TEST_BOOL(readline 3, (0 == rpRdr->readLine(line) && line.empty()))
	OT_TEST_BOOL(readline 4, (BufferedReader::EndOfFile == rpRdr->readLine(line) && line.empty()))

OT_END_TESTS(`BufferedReader')

