//
// ElCel Technology Test Harness
//
// Class: FileInputStream
//

include(../common.m4)

OT_TEST_INIT

#`include' "ot/io/FileInputStream.h"
#`include' "ot/io/FileOutputStream.h"
#`include' "ot/io/File.h"
#`include' "ot/io/IOException.h"
#`include' "ot/io/FileNotFoundException.h"

using namespace ot::io;

OT_START_TESTS(`FileInputStream')

	//
	// Basic API tests
	//

	File badFile1(OT_T(""));
	File badFile2(OT_T("."));
	File testFile(OT_T("test.out"));

	OT_TEST_THROW(badFile1, FileInputStream x(badFile1), IOException)
	OT_TEST_THROW(badFile2, FileInputStream x(badFile2), IOException)
	OT_TEST_THROW(testFile, FileInputStream x(testFile), FileNotFoundException)

	//
	// create a small file to work on
	//
	Byte buffer[3] = {1, 2, 3};
	RefPtr<FileOutputStream> rpOut;
	OT_TEST_VOID(new1, rpOut = new FileOutputStream(testFile))
	if(rpOut)
	{
		OT_TEST_VOID(write, rpOut->write(buffer, sizeof(buffer)))
		OT_TEST_VOID(close1, rpOut->close())
		OT_TEST_BOOL(exists, testFile.exists())
		OT_TEST_BOOL(length, testFile.length()==sizeof(buffer))
	}

	RefPtr<FileInputStream> rpTest;
	OT_TEST_VOID(new2, rpTest = new FileInputStream(testFile))
	if(rpTest)
	{
		// OpenTop API states it returns zero for available on FileInputStream
		OT_TEST_BOOL(available, rpTest->available()==0)
		OT_TEST_BOOL(markSupported, rpTest->markSupported()==false)
		OT_TEST_THROW(mark, rpTest->mark(1), IOException)
		OT_TEST_THROW(reset, rpTest->reset(), IOException)

		Byte input[3];

		OT_TEST_BOOL(read1, rpTest->read()==1)
		OT_TEST_BOOL(read2, rpTest->read(input+1, 2)==2)
		OT_TEST_BOOL(read3, *(input+1)==2)
		OT_TEST_BOOL(skip, rpTest->skip(1)==0)
		OT_TEST_BOOL(eof, rpTest->read()==InputStream::EndOfFile)
		OT_TEST_VOID(close2, rpTest->close())
	}

	// Make the file read-only then read it again
	OT_TEST_VOID(setReadOnly, testFile.setReadOnly(true));
	OT_TEST_VOID(open stream, rpTest.release(); rpTest = new FileInputStream(testFile))
	if(rpTest)
	{
		OT_TEST_BOOL(read4, rpTest->read()==1)
		OT_TEST_VOID(close3, rpTest->close())
	}

	// Make the file writable then delete it
	OT_TEST_VOID(setReadOnly, testFile.setReadOnly(false));
	OT_TEST_VOID(delete, testFile.deleteFile())

OT_END_TESTS(`FileInputStream')

