//
// ElCel Technology Test Harness
//
// Class: FileOutputStream
//





#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/io/FileOutputStream.h"
#include "ot/io/File.h"
#include "ot/io/IOException.h"
#include "ot/io/FileNotFoundException.h"

using namespace ot::io;


void FileOutputStream_Tests()
{
	testMessage(OT_T("Starting tests for FileOutputStream"));

	//
	// Basic API tests
	//

	File badFile1(OT_T(""));
	File badFile2(OT_T("."));
	File testFile(OT_T("test.out"));

	Byte buffer[3] = {1, 2, 3};

	try
	{
		FileOutputStream x(badFile1);
		testFailed(OT_T("badFile1"));
	}
	catch(IOException& e)
	{
		goodCatch(OT_T("badFile1"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("badFile1"));
	}
	try
	{
		FileOutputStream x(badFile2);
		testFailed(OT_T("badFile2"));
	}
	catch(IOException& e)
	{
		goodCatch(OT_T("badFile2"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("badFile2"));
	}

	try
	{
		if(testFile.exists()==false) {testPassed(OT_T("exists"));} else {testFailed(OT_T("exists"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("exists"));
	}
	
	RefPtr<FileOutputStream> rpTest = new FileOutputStream(testFile);
	try
	{
		rpTest->write(buffer[0]); testPassed(OT_T("write"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->write(buffer+1, 2); testPassed(OT_T("write"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->close(); testPassed(OT_T("close"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("close"));
	}

	try
	{
		if(testFile.exists()) {testPassed(OT_T("exists"));} else {testFailed(OT_T("exists"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("exists"));
	}
	try
	{
		if(testFile.length()==3) {testPassed(OT_T("length"));} else {testFailed(OT_T("length"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("length"));
	}

	// now append to the file
	rpTest = new FileOutputStream(testFile.getPath(), true);
	try
	{
		rpTest->write(4); testPassed(OT_T("write"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->close(); testPassed(OT_T("close"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("close"));
	}

	try
	{
		if(testFile.length()==4) {testPassed(OT_T("length"));} else {testFailed(OT_T("length"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("length"));
	}

	// delete the file, ready for next time
	try
	{
		testFile.deleteFile(); testPassed(OT_T("delete"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("delete"));
	}


	testMessage(OT_T("End of tests for FileOutputStream"));
}

