//
// ElCel Technology Test Harness
//
// Class: InputStreamReader
//

include(../common.m4)

OT_TEST_INIT

#`include' "ot/io/InputStreamReader.h"
#`include' "ot/io/FileInputStream.h"
#`include' "ot/io/FileOutputStream.h"
#`include' "ot/io/File.h"
#`include' "ot/io/IOException.h"
#`include' "ot/io/UnsupportedEncodingException.h"
#`include' "ot/base/NullPointerException.h"

using namespace ot::io;

OT_START_TESTS(`InputStreamReader')

	//
	// Basic API tests
	//

	File testFile(OT_T("test.out"));

	//
	// create a small file to work on
	//
	Byte buffer[3] = {'a', 'b', 'c'};
	RefPtr<FileOutputStream> rpOut = new FileOutputStream(testFile);
	OT_TEST_VOID(write, rpOut->write(buffer, sizeof(buffer)))
	OT_TEST_VOID(close, rpOut->close())
	OT_TEST_BOOL(exists, testFile.exists())
	OT_TEST_BOOL(length, testFile.length()==sizeof(buffer))

	RefPtr<FileInputStream> rpIn = new FileInputStream(testFile);

	OT_TEST_THROW(unsupported, InputStreamReader x(rpIn.get(), OT_T("qwertyuiop")), UnsupportedEncodingException)
	OT_TEST_THROW(null, InputStreamReader x(0), NullPointerException)
	OT_TEST_THROW(null, InputStreamReader x(rpIn.get(), (CodeConverter*)0), NullPointerException)

	RefPtr<InputStreamReader> rpTest = new InputStreamReader(rpIn.get(), OT_T("us-ascii"));

	OT_TEST_BOOL(markSupported, rpTest->markSupported()==false)
	OT_TEST_THROW(mark, rpTest->mark(1), IOException)
	OT_TEST_THROW(reset, rpTest->reset(), IOException)

	CharType input[3];

	OT_TEST_BOOL(read, rpTest->read()=='a')
	OT_TEST_BOOL(read, rpTest->read(input+1, 2)==2)
	OT_TEST_BOOL(read, *(input+1)=='b')
	OT_TEST_BOOL(skip, rpTest->skip(1)==0)
	OT_TEST_BOOL(eof, rpTest->read()==InputStream::EndOfFile)
	OT_TEST_BOOL(eof, rpTest->readAtomic().isEOF())
	OT_TEST_VOID(close, rpTest->close())

	// delete the file, ready for next time
	OT_TEST_VOID(delete, testFile.deleteFile())

OT_END_TESTS(`InputStreamReader')

