//
// ElCel Technology Test Harness
//
// Class: OutputStreamWriter
//





#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/io/OutputStreamWriter.h"
#include "ot/io/FileInputStream.h"
#include "ot/io/FileOutputStream.h"
#include "ot/io/File.h"
#include "ot/io/IOException.h"
#include "ot/io/UnsupportedEncodingException.h"
#include "ot/base/NullPointerException.h"

using namespace ot::io;


void OutputStreamWriter_Tests()
{
	testMessage(OT_T("Starting tests for OutputStreamWriter"));

	//
	// Basic API tests
	//

	File testFile(OT_T("test.out"));

	//
	// create a small output file
	//
	CharType buffer[3] = {'a', 'b', 'c'};
	RefPtr<FileOutputStream> rpOut = new FileOutputStream(testFile);

	try
	{
		OutputStreamWriter x(rpOut.get(), OT_T("qwertyuiop"));
		testFailed(OT_T("unsupported"));
	}
	catch(UnsupportedEncodingException& e)
	{
		goodCatch(OT_T("unsupported"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("unsupported"));
	}
	try
	{
		OutputStreamWriter x(0);
		testFailed(OT_T("null"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("null"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("null"));
	}
	try
	{
		OutputStreamWriter x(rpOut.get(), (CodeConverter*)0);
		testFailed(OT_T("null"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("null"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("null"));
	}

	RefPtr<OutputStreamWriter> rpTest = new OutputStreamWriter(rpOut.get(), OT_T("us-ascii"));

	try
	{
		rpTest->write(0, sizeof(buffer)/sizeof(CharType));
		testFailed(OT_T("write"));
	}
	catch(NullPointerException& e)
	{
		goodCatch(OT_T("write"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->write(buffer, 0); testPassed(OT_T("write"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->write(buffer, sizeof(buffer)/sizeof(CharType)); testPassed(OT_T("write"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->write('d'); testPassed(OT_T("write"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("write"));
	}
	try
	{
		rpTest->close(); testPassed(OT_T("close"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("close"));
	}
	try
	{
		if(testFile.length()==4) {testPassed(OT_T("length"));} else {testFailed(OT_T("length"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("length"));
	}

	// delete the file, ready for next time
	try
	{
		testFile.deleteFile(); testPassed(OT_T("delete"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("delete"));
	}


	testMessage(OT_T("End of tests for OutputStreamWriter"));
}

