//
// ElCel Technology Test Harness
//
// Class: OutputStreamWriter
//

include(../common.m4)

OT_TEST_INIT

#`include' "ot/io/OutputStreamWriter.h"
#`include' "ot/io/FileInputStream.h"
#`include' "ot/io/FileOutputStream.h"
#`include' "ot/io/File.h"
#`include' "ot/io/IOException.h"
#`include' "ot/io/UnsupportedEncodingException.h"
#`include' "ot/base/NullPointerException.h"

using namespace ot::io;

OT_START_TESTS(`OutputStreamWriter')

	//
	// Basic API tests
	//

	File testFile(OT_T("test.out"));

	//
	// create a small output file
	//
	CharType buffer[3] = {'a', 'b', 'c'};
	RefPtr<FileOutputStream> rpOut = new FileOutputStream(testFile);

	OT_TEST_THROW(unsupported, OutputStreamWriter x(rpOut.get(), OT_T("qwertyuiop")), UnsupportedEncodingException)
	OT_TEST_THROW(null, OutputStreamWriter x(0), NullPointerException)
	OT_TEST_THROW(null, OutputStreamWriter x(rpOut.get(), (CodeConverter*)0), NullPointerException)

	RefPtr<OutputStreamWriter> rpTest = new OutputStreamWriter(rpOut.get(), OT_T("us-ascii"));

	OT_TEST_THROW(write, rpTest->write(0, sizeof(buffer)/sizeof(CharType)), NullPointerException)
	OT_TEST_VOID(write, rpTest->write(buffer, 0))
	OT_TEST_VOID(write, rpTest->write(buffer, sizeof(buffer)/sizeof(CharType)))
	OT_TEST_VOID(write, rpTest->write('d'))
	OT_TEST_VOID(close, rpTest->close())
	OT_TEST_BOOL(length, testFile.length()==4)

	// delete the file, ready for next time
	OT_TEST_VOID(delete, testFile.deleteFile())

OT_END_TESTS(`OutputStreamWriter')

