//
// ElCel Technology Test Harness 
//
// Class: HttpClient
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/net/URL.h"
#include "ot/net/HttpURLConnection.h"
#include "ot/io/InputStream.h"

using namespace ot::io; 
using namespace ot::net; 


void HttpClient_Tests()
{
	testMessage(OT_T("Starting tests for HttpClient"));

	String server = getTestAttribute(OT_T("server"));
	if(server.empty())
		server = OT_T("www.elcel.com/");

	server.insert(0, OT_T("http://"));

	RefPtr<InputStream> rpIS;
	RefPtr<HttpURLConnection> rpUrlConn;
	URL url1(server); 

	// Test accessing the stream directly
	try
	{
		rpIS = url1.openStream(); testPassed(OT_T("openStream"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("openStream"));
	}
	if(rpIS)
	{
		try
	{
		if(rpIS && rpIS->read() == '<') {testPassed(OT_T("checkStream"));} else {testFailed(OT_T("checkStream"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("checkStream"));
	}

		// Test accessing the URLConnection
		try
	{
		rpUrlConn = static_cast<HttpURLConnection*>(url1.openConnection().get()); testPassed(OT_T("openURLConnection"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("openURLConnection"));
	}
		try
	{
		if(rpUrlConn) {testPassed(OT_T("connection returned"));} else {testFailed(OT_T("connection returned"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("connection returned"));
	}
		if(rpUrlConn)
		{
			try
	{
		rpUrlConn->connect(); testPassed(OT_T("connect"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("connect"));
	}
			long resourceSize =  rpUrlConn->getContentLength();
			try
	{
		if(resourceSize != -1) {testPassed(OT_T("checkLength"));} else {testFailed(OT_T("checkLength"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("checkLength"));
	}
			try
	{
		if(rpUrlConn->isConnected()) {testPassed(OT_T("isConnected"));} else {testFailed(OT_T("isConnected"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("isConnected"));
	}
			if(rpUrlConn->isConnected())
			{
				try
	{
		if(rpUrlConn->getResponseCode() == HttpURLConnection::HTTP_OK) {testPassed(OT_T("checkResponse"));} else {testFailed(OT_T("checkResponse"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("checkResponse"));
	}
				try
	{
		if(rpUrlConn->getDate().isValid()) {testPassed(OT_T("getDate"));} else {testFailed(OT_T("getDate"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getDate"));
	}

				try
	{
		rpIS = rpUrlConn->getInputStream(); testPassed(OT_T("getInputStream"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getInputStream"));
	}
				try
	{
		if(rpIS) {testPassed(OT_T("valid input stream"));} else {testFailed(OT_T("valid input stream"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("valid input stream"));
	}
				if(rpIS)
				{
					try
	{
		if(rpIS->read() == '<') {testPassed(OT_T("checkStream2"));} else {testFailed(OT_T("checkStream2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("checkStream2"));
	}
					try
	{
		if(rpIS->skip(resourceSize-1) == (size_t)resourceSize-1) {testPassed(OT_T("skip"));} else {testFailed(OT_T("skip"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("skip"));
	}
					try
	{
		if(rpIS->read() == InputStream::EndOfFile) {testPassed(OT_T("eof"));} else {testFailed(OT_T("eof"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("eof"));
	}
				}
			}
		}
	}



	testMessage(OT_T("End of tests for HttpClient"));
}

