//
// ElCel Technology Test Harness 
//
// Class: Socket
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/net/InetAddress.h"
#include "ot/net/Socket.h"
#include "ot/net/SocketException.h"

using namespace ot::net; 


void Socket_Tests()
{
	testMessage(OT_T("Starting tests for Socket"));

	const int wwwPort = 80;
	String server = getTestAttribute(OT_T("server"));
	if(server.empty())
		server = OT_T("www.elcel.com");

	//
	// Test socket processing by connecting to a well-knwon HTTP server
	//
	RefPtr<InetAddress> rpAddr = InetAddress::GetByName(server);
	RefPtr<Socket> rpSocket = new Socket;
	try
	{
		if(!rpSocket->isConnected()) {testPassed(OT_T("isConnected"));} else {testFailed(OT_T("isConnected"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("isConnected"));
	}
	try
	{
		rpSocket->connect(rpAddr.get(), wwwPort); testPassed(OT_T("connect"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("connect"));
	}
	try
	{
		rpSocket->connect(InetAddress::GetByName(server).get(), wwwPort);
		testFailed(OT_T("(re)connect"));
	}
	catch(SocketException& e)
	{
		goodCatch(OT_T("(re)connect"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("(re)connect"));
	}
	try
	{
		if(rpSocket->isConnected()) {testPassed(OT_T("isConnected1"));} else {testFailed(OT_T("isConnected1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("isConnected1"));
	}
	try
	{
		if(rpSocket->getPort()==wwwPort) {testPassed(OT_T("getPort"));} else {testFailed(OT_T("getPort"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getPort"));
	}
	try
	{
		if(rpSocket->getInetAddress() && rpSocket->getInetAddress()->equals(*rpAddr)) {testPassed(OT_T("getInetAddress"));} else {testFailed(OT_T("getInetAddress"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getInetAddress"));
	}

	//
	// Close the socket, then check for closed exceptions
	//
	try
	{
		rpSocket->close(); testPassed(OT_T("close"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("close"));
	}
	try
	{
		if(!rpSocket->isConnected()) {testPassed(OT_T("isConnected2"));} else {testFailed(OT_T("isConnected2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("isConnected2"));
	}



	testMessage(OT_T("End of tests for Socket"));
}

