//
// ElCel Technology Test Harness 
//
// Class: URL
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/base/String.h"
#include "ot/base/StringUtils.h"
#include "ot/net/URL.h"
#include "ot/net/MalformedURLException.h"

using namespace ot::net; 

void testURL(const String& sURL,
             const String& sContext,
             const String& protocol,
             const String& host,
             const String& file,
             const String& ref,
             const String& query,
             const String& userInfo,
             int port)
{
	URL url;
	if(!sContext.empty())
	{
		URL urlContext(sContext);
		url = URL(urlContext, sURL);
	}
	else
	{
		url = URL(sURL);
	}

	URL url2(url.toExternalForm());
	try
	{
		if(url == url2) {testPassed(OT_T("roundTrip"));} else {testFailed(OT_T("roundTrip"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("roundTrip"));
	}

	try
	{
		if(url.getProtocol() == protocol) {testPassed(OT_T("getProtocol"));} else {testFailed(OT_T("getProtocol"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getProtocol"));
	}
	try
	{
		if(url.getHost() == host) {testPassed(OT_T("gethost"));} else {testFailed(OT_T("gethost"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("gethost"));
	}
	try
	{
		if(url.getFile() == file) {testPassed(OT_T("getFile"));} else {testFailed(OT_T("getFile"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getFile"));
	}
	//testMessage(url.getFile());
	try
	{
		if(url.getRef() == ref) {testPassed(OT_T("getRef"));} else {testFailed(OT_T("getRef"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getRef"));
	}
	try
	{
		if(url.getPort() == port) {testPassed(OT_T("getPort"));} else {testFailed(OT_T("getPort"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getPort"));
	}
	try
	{
		if(url.getQuery() == query) {testPassed(OT_T("getQuery"));} else {testFailed(OT_T("getQuery"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getQuery"));
	}
	try
	{
		if(url.getUserInfo() == userInfo) {testPassed(OT_T("getUserInfo"));} else {testFailed(OT_T("getUserInfo"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getUserInfo"));
	}
}


void URL_Tests()
{
	testMessage(OT_T("Starting tests for URL"));

	String s1 = OT_T("http://a.b.c/d/e#f");
	String sNull;

	try
	{
		testURL(s1, sNull, OT_T("http"), OT_T("a.b.c"), OT_T("/d/e"), OT_T("f"), sNull, sNull, -1); testPassed(OT_T("s1"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s1"));
	}
	String s1a = OT_T("file#ref");
	try
	{
		testURL(s1a, s1, OT_T("http"), OT_T("a.b.c"), OT_T("/d/file"), OT_T("ref"), sNull, sNull, -1); testPassed(OT_T("s1a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s1a"));
	}

	String s2 = OT_T("http://www.elcel.com:8080/test.cgi?query+string#ref");
	try
	{
		testURL(s2, sNull, OT_T("http"), OT_T("www.elcel.com"), OT_T("/test.cgi?query+string"), OT_T("ref"), OT_T("query+string"), sNull, 8080); testPassed(OT_T("s2"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s2"));
	}
	String s2a = OT_T("test2.cgi");
	try
	{
		testURL(s2a, s2, OT_T("http"), OT_T("www.elcel.com"), OT_T("/test2.cgi"), sNull, sNull, sNull, 8080); testPassed(OT_T("s2a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s2a"));
	}
	String s2b = OT_T("test2b.html#xxx");
	try
	{
		testURL(s2b, s2, OT_T("http"), OT_T("www.elcel.com"), OT_T("/test2b.html"), OT_T("xxx"), sNull, sNull, 8080); testPassed(OT_T("s2b"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s2b"));
	}

	String s3 = OT_T("ftp://user:password@ftp.elcel.com/data/test.txt");
	try
	{
		testURL(s3, sNull, OT_T("ftp"), OT_T("ftp.elcel.com"), OT_T("/data/test.txt"), sNull, sNull, OT_T("user:password"), -1); testPassed(OT_T("s3"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s3"));
	}
	String s3a = OT_T("test2.txt");
	try
	{
		testURL(s3a, s3, OT_T("ftp"), OT_T("ftp.elcel.com"), OT_T("/data/test2.txt"), sNull, sNull, OT_T("user:password"), -1); testPassed(OT_T("s3a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s3a"));
	}
	String s3b = OT_T("ftp://ftp.elcel.com:1234/data/test2.txt");
	try
	{
		testURL(s3b, s3, OT_T("ftp"), OT_T("ftp.elcel.com"), OT_T("/data/test2.txt"), sNull, sNull, sNull, 1234); testPassed(OT_T("s3b"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s3b"));
	}

	String s4 = OT_T("XXXX://www.elcel.com/");
	try
	{
		URL url3(s4);
		testFailed(OT_T("malformed url"));
	}
	catch(MalformedURLException& e)
	{
		goodCatch(OT_T("malformed url"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("malformed url"));
	}

	URL comp1(OT_T("http://www.elcel.com/index.html"));
	URL comp2(OT_T("HTTP://WWW.ELCEL.COM:80/index.html"));
	URL comp2a(OT_T("HTTP://WWW.ELCEL.COM:8080/index.html"));
	URL comp2b(OT_T("HTTP://WWW.ELCEL.COM/"));
	URL comp3(OT_T("HTTP://ELCEL.COM/index.html"));
	try
	{
		if(comp1 == comp2) {testPassed(OT_T("equals"));} else {testFailed(OT_T("equals"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("equals"));
	}
	try
	{
		if(comp2 != comp2a) {testPassed(OT_T("equals"));} else {testFailed(OT_T("equals"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("equals"));
	}
	try
	{
		if(comp2 != comp2b) {testPassed(OT_T("equals"));} else {testFailed(OT_T("equals"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("equals"));
	}
	try
	{
		if(comp1 == comp3) {testPassed(OT_T("equals"));} else {testFailed(OT_T("equals"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("equals"));
	}

#ifdef WIN32
	String prefix;
#else
	String prefix = OT_T("/");
#endif

	String s5 = OT_T("file:///c:\\temp\\x.txt");
	try
	{
		testURL(s5, sNull, OT_T("file"), sNull, prefix + OT_T("c:/temp/x.txt"), sNull, sNull, sNull, -1); testPassed(OT_T("s5"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s5"));
	}
	String s5a = OT_T("test2.txt");
	try
	{
		testURL(s5a, s5, OT_T("file"), sNull, prefix + OT_T("c:/temp/test2.txt"), sNull, sNull, sNull, -1); testPassed(OT_T("s5a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s5a"));
	}

	String s6 = OT_T("file://host/share/rootdir/filename");
	try
	{
		testURL(s6, sNull, OT_T("file"), OT_T("host"), OT_T("/share/rootdir/filename"), sNull, sNull, sNull, -1); testPassed(OT_T("s6"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s6"));
	}
	String s6a = OT_T("test2.txt");
	try
	{
		testURL(s6a, s6, OT_T("file"), OT_T("host"), OT_T("/share/rootdir/test2.txt"), sNull, sNull, sNull, -1); testPassed(OT_T("s6a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s6a"));
	}

	String s7 = OT_T("http://www.elcel.com:80/dir1/dir2/search?xml");
	String s7a = OT_T("../../images/image1.gif");
	String s7b = OT_T("../../../images/image1.gif");
	try
	{
		testURL(s7a, s7, OT_T("http"), OT_T("www.elcel.com"), OT_T("/images/image1.gif"), sNull, sNull, sNull, 80); testPassed(OT_T("s7a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s7a"));
	}
	try
	{
		testURL(s7b, s7, OT_T("http"), OT_T("www.elcel.com"), OT_T("/../images/image1.gif"), sNull, sNull, sNull, 80); testPassed(OT_T("s7b"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s7b"));
	}

	String s8 = OT_T("file:tests/test.xml");
	String s8a = OT_T("../../test2.xml");
	try
	{
		testURL(s8a, s8, OT_T("file"), sNull, OT_T("../test2.xml"), sNull, sNull, sNull, -1); testPassed(OT_T("s8a"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("s8a"));
	}


	testMessage(OT_T("End of tests for URL"));
}

