//
// ElCel Technology Test Harness 
//
// Class: URL
//

include(../common.m4)
OT_TEST_INIT

`#include "ot/base/String.h"'
`#include "ot/base/StringUtils.h"'
`#include "ot/net/URL.h"'
`#include "ot/net/MalformedURLException.h"'

using namespace ot::net; 

void testURL(const String& sURL,
             const String& sContext,
             const String& protocol,
             const String& host,
             const String& file,
             const String& ref,
             const String& query,
             const String& userInfo,
             int port)
{
	URL url;
	if(!sContext.empty())
	{
		URL urlContext(sContext);
		url = URL(urlContext, sURL);
	}
	else
	{
		url = URL(sURL);
	}

	URL url2(url.toExternalForm());
	OT_TEST_BOOL(roundTrip, url == url2)

	OT_TEST_BOOL(getProtocol, url.getProtocol() == protocol)
	OT_TEST_BOOL(gethost, url.getHost() == host)
	OT_TEST_BOOL(getFile, url.getFile() == file)
	//testMessage(url.getFile());
	OT_TEST_BOOL(getRef, url.getRef() == ref)
	OT_TEST_BOOL(getPort, url.getPort() == port)
	OT_TEST_BOOL(getQuery, url.getQuery() == query)
	OT_TEST_BOOL(getUserInfo, url.getUserInfo() == userInfo)
}

OT_START_TESTS(`URL')

	String s1 = OT_T("http://a.b.c/d/e#f");
	String sNull;

	OT_TEST_VOID(s1, testURL(s1, sNull, OT_T("http"), OT_T("a.b.c"), OT_T("/d/e"), OT_T("f"), sNull, sNull, -1))
	String s1a = OT_T("file#ref");
	OT_TEST_VOID(s1a, testURL(s1a, s1, OT_T("http"), OT_T("a.b.c"), OT_T("/d/file"), OT_T("ref"), sNull, sNull, -1))

	String s2 = OT_T("http://www.elcel.com:8080/test.cgi?query+string#ref");
	OT_TEST_VOID(s2, testURL(s2, sNull, OT_T("http"), OT_T("www.elcel.com"), OT_T("/test.cgi?query+string"), OT_T("ref"), OT_T("query+string"), sNull, 8080))
	String s2a = OT_T("test2.cgi");
	OT_TEST_VOID(s2a, testURL(s2a, s2, OT_T("http"), OT_T("www.elcel.com"), OT_T("/test2.cgi"), sNull, sNull, sNull, 8080))
	String s2b = OT_T("test2b.html#xxx");
	OT_TEST_VOID(s2b, testURL(s2b, s2, OT_T("http"), OT_T("www.elcel.com"), OT_T("/test2b.html"), OT_T("xxx"), sNull, sNull, 8080))

	String s3 = OT_T("ftp://user:password@ftp.elcel.com/data/test.txt");
	OT_TEST_VOID(s3, testURL(s3, sNull, OT_T("ftp"), OT_T("ftp.elcel.com"), OT_T("/data/test.txt"), sNull, sNull, OT_T("user:password"), -1))
	String s3a = OT_T("test2.txt");
	OT_TEST_VOID(s3a, testURL(s3a, s3, OT_T("ftp"), OT_T("ftp.elcel.com"), OT_T("/data/test2.txt"), sNull, sNull, OT_T("user:password"), -1))
	String s3b = OT_T("ftp://ftp.elcel.com:1234/data/test2.txt");
	OT_TEST_VOID(s3b, testURL(s3b, s3, OT_T("ftp"), OT_T("ftp.elcel.com"), OT_T("/data/test2.txt"), sNull, sNull, sNull, 1234))

	String s4 = OT_T("XXXX://www.elcel.com/");
	OT_TEST_THROW(malformed url, URL url3(s4), MalformedURLException)

	URL comp1(OT_T("http://www.elcel.com/index.html"));
	URL comp2(OT_T("HTTP://WWW.ELCEL.COM:80/index.html"));
	URL comp2a(OT_T("HTTP://WWW.ELCEL.COM:8080/index.html"));
	URL comp2b(OT_T("HTTP://WWW.ELCEL.COM/"));
	URL comp3(OT_T("HTTP://ELCEL.COM/index.html"));
	OT_TEST_BOOL(equals, comp1 == comp2)
	OT_TEST_BOOL(equals, comp2 != comp2a)
	OT_TEST_BOOL(equals, comp2 != comp2b)
	OT_TEST_BOOL(equals, comp1 == comp3)

`#ifdef WIN32'
	String prefix;
`#else'
	String prefix = OT_T("/");
`#endif'

	String s5 = OT_T("file:///c:\\temp\\x.txt");
	OT_TEST_VOID(s5, testURL(s5, sNull, OT_T("file"), sNull, prefix + OT_T("c:/temp/x.txt"), sNull, sNull, sNull, -1))
	String s5a = OT_T("test2.txt");
	OT_TEST_VOID(s5a, testURL(s5a, s5, OT_T("file"), sNull, prefix + OT_T("c:/temp/test2.txt"), sNull, sNull, sNull, -1))

	String s6 = OT_T("file://host/share/rootdir/filename");
	OT_TEST_VOID(s6, testURL(s6, sNull, OT_T("file"), OT_T("host"), OT_T("/share/rootdir/filename"), sNull, sNull, sNull, -1))
	String s6a = OT_T("test2.txt");
	OT_TEST_VOID(s6a, testURL(s6a, s6, OT_T("file"), OT_T("host"), OT_T("/share/rootdir/test2.txt"), sNull, sNull, sNull, -1))

	String s7 = OT_T("http://www.elcel.com:80/dir1/dir2/search?xml");
	String s7a = OT_T("../../images/image1.gif");
	String s7b = OT_T("../../../images/image1.gif");
	OT_TEST_VOID(s7a, testURL(s7a, s7, OT_T("http"), OT_T("www.elcel.com"), OT_T("/images/image1.gif"), sNull, sNull, sNull, 80))
	OT_TEST_VOID(s7b, testURL(s7b, s7, OT_T("http"), OT_T("www.elcel.com"), OT_T("/../images/image1.gif"), sNull, sNull, sNull, 80))

	String s8 = OT_T("file:tests/test.xml");
	String s8a = OT_T("../../test2.xml");
	OT_TEST_VOID(s8a, testURL(s8a, s8, OT_T("file"), sNull, OT_T("../test2.xml"), sNull, sNull, sNull, -1))

OT_END_TESTS(`URL')

