//
// ElCel Technology Test Harness 
//
// Class: AttributesImpl
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/sax/XMLReaderFactory.h"
#include "ot/sax/XMLReader.h"
#include "ot/sax/AttributesImpl.h"
#include "ot/base/NumUtils.h"
#include "ot/base/IllegalArgumentException.h"

#include <stdio.h>

using namespace ot; 
using namespace ot::sax; 
using namespace ot::io; 


void AttributesImpl_Tests()
{
	testMessage(OT_T("Starting tests for AttributesImpl"));

	const size_t iNumAttributes = 10;
	String value;

	AttributesImpl impl1;
	size_t i;
	for(i = 0; i < iNumAttributes; i++)
	{
		String qnameStr = String(OT_T("ot:attribute")) + NumUtils::ToString(i);
		try
	{
		impl1.addAttribute(OT_T("http://opentop.org/ns"), qnameStr, OT_T("CDATA"), OT_T("hello")); testPassed(OT_T("addAttribute"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("addAttribute"));
	}
		try
	{
		if(impl1.getLength() == i+1) {testPassed(OT_T("getLength"));} else {testFailed(OT_T("getLength"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength"));
	}
	}

	AttributesImpl impl2(impl1);
	try
	{
		if(impl2.getLength() == iNumAttributes) {testPassed(OT_T("getLength2"));} else {testFailed(OT_T("getLength2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength2"));
	}

	// test that setAttributes works
	AttributesImpl impl3;
	try
	{
		impl3.setAttributes(impl2); testPassed(OT_T("setAttributes"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setAttributes"));
	}
	try
	{
		if(impl2.getLength() == iNumAttributes) {testPassed(OT_T("getLength3"));} else {testFailed(OT_T("getLength3"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength3"));
	}

	// test that clear works
	try
	{
		impl3.clear(); testPassed(OT_T("clear"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("clear"));
	}
	try
	{
		if(impl3.getLength() == 0) {testPassed(OT_T("getLength4"));} else {testFailed(OT_T("getLength4"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength4"));
	}

	try
	{
		impl3.setAttributes(impl2); testPassed(OT_T("setAttributes2"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setAttributes2"));
	}
	try
	{
		if(impl3.getLength() == impl2.getLength()) {testPassed(OT_T("getLength5"));} else {testFailed(OT_T("getLength5"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength5"));
	}

	for(i = 1; i<=iNumAttributes; i++)
	{
		try
	{
		impl3.removeAttribute(iNumAttributes-i); testPassed(OT_T("removeAttribute"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("removeAttribute"));
	}
		try
	{
		if(impl3.getLength() == iNumAttributes-i) {testPassed(OT_T("getLength6"));} else {testFailed(OT_T("getLength6"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength6"));
	}
		try
	{
		impl3.removeAttribute(iNumAttributes-i);
		testFailed(OT_T("removeAttribute throw"));
	}
	catch(ot::IllegalArgumentException& e)
	{
		goodCatch(OT_T("removeAttribute throw"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("removeAttribute throw"));
	}
	}

	try
	{
		impl3.setAttributes(impl2); testPassed(OT_T("setAttributes3"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setAttributes3"));
	}
	try
	{
		if(impl3.getLength() == iNumAttributes) {testPassed(OT_T("getLength7"));} else {testFailed(OT_T("getLength7"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLength7"));
	}
	try
	{
		if(impl3.getLocalName(1) == OT_T("attribute1")) {testPassed(OT_T("getLocalName"));} else {testFailed(OT_T("getLocalName"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLocalName"));
	}
	// test that an invalid index returns an empty string
	try
	{
		if(impl3.getLocalName(iNumAttributes) == OT_T("")) {testPassed(OT_T("getLocalName2"));} else {testFailed(OT_T("getLocalName2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getLocalName2"));
	}

	// test that setAttribute throws when you supply an
	// index off the end of the array
	try
	{
		impl3.setAttribute(iNumAttributes, OT_T("newuri"), OT_T("ot:newqname"), OT_T("newtype"), OT_T("goodbye"));
		testFailed(OT_T("setAttribute throw"));
	}
	catch(ot::IllegalArgumentException& e)
	{
		goodCatch(OT_T("setAttribute throw"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setAttribute throw"));
	}
	// set the last element to have new values
	try
	{
		impl3.setAttribute(0, OT_T("newuri"), OT_T("ot:newqname"), OT_T("newtype"), OT_T("goodbye")); testPassed(OT_T("setAttribute2"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setAttribute2"));
	}
	try
	{
		if(impl3.getValue(0) == OT_T("goodbye")) {testPassed(OT_T("getValue"));} else {testFailed(OT_T("getValue"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getValue"));
	}

	// test that setQName throws when you supply an index off the end of the array
	try
	{
		impl3.setQName(iNumAttributes, OT_T("ot:new"));
		testFailed(OT_T("setQName throw"));
	}
	catch(ot::IllegalArgumentException& e)
	{
		goodCatch(OT_T("setQName throw"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setQName throw"));
	}

	// test that setQName works
	try
	{
		impl3.setQName(0, OT_T("ot:new")); testPassed(OT_T("setQName"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setQName"));
	}
	try
	{
		if(impl3.getQName(0) == OT_T("ot:new")) {testPassed(OT_T("getQName"));} else {testFailed(OT_T("getQName"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getQName"));
	}

	// test that getIndex (qname) works
	for (i = 0; i < iNumAttributes; i++)
	{
		String qnameStr = String(OT_T("ot:attribute")) + NumUtils::ToString(i);
		try
	{
		if(impl1.getIndex(qnameStr) == (int)i) {testPassed(OT_T("getIndex qname"));} else {testFailed(OT_T("getIndex qname"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getIndex qname"));
	}
	}

	// test that getIndex (qname) returns -1 if it cant find the attribute
	try
	{
		if(impl1.getIndex(OT_T("nonexistent")) == -1) {testPassed(OT_T("getIndex qname returns -1"));} else {testFailed(OT_T("getIndex qname returns -1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getIndex qname returns -1"));
	}
	try
	{
		if(impl3.getIndex(OT_T("nosuchuri"), OT_T("nosuchname")) == -1) {testPassed(OT_T("getIndex"));} else {testFailed(OT_T("getIndex"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getIndex"));
	}

	// test that setType works
	try
	{
		impl3.setType(0, OT_T("ID")); testPassed(OT_T("setType"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setType"));
	}
	try
	{
		if(impl3.getType(0) == OT_T("ID")) {testPassed(OT_T("getType"));} else {testFailed(OT_T("getType"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getType"));
	}
	// check setType throws
	try
	{
		impl3.setType(iNumAttributes, OT_T("ID"));
		testFailed(OT_T("setType throw"));
	}
	catch(ot::IllegalArgumentException& e)
	{
		goodCatch(OT_T("setType throw"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setType throw"));
	}

	// test that setURI throws
	try
	{
		impl3.setURI(iNumAttributes, OT_T("http://opentop.org/ns"));
		testFailed(OT_T("setURI throw"));
	}
	catch(ot::IllegalArgumentException& e)
	{
		goodCatch(OT_T("setURI throw"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setURI throw"));
	}
	
	// test that setValue works
	try
	{
		impl3.setAttributes(impl2); testPassed(OT_T("setAttributes"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setAttributes"));
	}
	try
	{
		impl3.setValue(0, OT_T("newvalue")); testPassed(OT_T("setValue"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setValue"));
	}
	try
	{
		if(impl3.getValue(0) == OT_T("newvalue")) {testPassed(OT_T("getValue"));} else {testFailed(OT_T("getValue"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("getValue"));
	}

	// test that setValue throws
	try
	{
		impl3.setValue(iNumAttributes, OT_T("newvalue"));
		testFailed(OT_T("setValue throw"));
	}
	catch(ot::IllegalArgumentException& e)
	{
		goodCatch(OT_T("setValue throw"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("setValue throw"));
	}


	testMessage(OT_T("End of tests for AttributesImpl"));
}
