//
// ElCel Technology Test Harness 
//
// Class: AttributesImpl
//

include(../common.m4)
OT_TEST_INIT

`#include "ot/sax/XMLReaderFactory.h"'
`#include "ot/sax/XMLReader.h"'
`#include "ot/sax/AttributesImpl.h"'
`#include "ot/base/NumUtils.h"'
`#include "ot/base/IllegalArgumentException.h"'

`#include <stdio.h>'

using namespace ot; 
using namespace ot::sax; 
using namespace ot::io; 

OT_START_TESTS(`AttributesImpl')

	const size_t iNumAttributes = 10;
	String value;

	AttributesImpl impl1;
	size_t i;
	for(i = 0; i < iNumAttributes; i++)
	{
		String qnameStr = String(OT_T("ot:attribute")) + NumUtils::ToString(i);
		OT_TEST_VOID(addAttribute, impl1.addAttribute(OT_T("http://opentop.org/ns"), qnameStr, OT_T("CDATA"), OT_T("hello")))
		OT_TEST_BOOL(getLength, impl1.getLength() == i+1)
	}

	AttributesImpl impl2(impl1);
	OT_TEST_BOOL(getLength2, impl2.getLength() == iNumAttributes)

	// test that setAttributes works
	AttributesImpl impl3;
	OT_TEST_VOID(setAttributes, impl3.setAttributes(impl2))
	OT_TEST_BOOL(getLength3, impl2.getLength() == iNumAttributes)

	// test that clear works
	OT_TEST_VOID(clear, impl3.clear())
	OT_TEST_BOOL(getLength4, impl3.getLength() == 0)

	OT_TEST_VOID(setAttributes2, impl3.setAttributes(impl2))
	OT_TEST_BOOL(getLength5, impl3.getLength() == impl2.getLength())

	for(i = 1; i<=iNumAttributes; i++)
	{
		OT_TEST_VOID(removeAttribute, impl3.removeAttribute(iNumAttributes-i))
		OT_TEST_BOOL(getLength6, impl3.getLength() == iNumAttributes-i)
		OT_TEST_THROW(removeAttribute throw, impl3.removeAttribute(iNumAttributes-i), ot::IllegalArgumentException)
	}

	OT_TEST_VOID(setAttributes3, impl3.setAttributes(impl2))
	OT_TEST_BOOL(getLength7, impl3.getLength() == iNumAttributes)
	OT_TEST_BOOL(getLocalName, impl3.getLocalName(1) == OT_T("attribute1"))
	// test that an invalid index returns an empty string
	OT_TEST_BOOL(getLocalName2, impl3.getLocalName(iNumAttributes) == OT_T(""))

	// test that setAttribute throws when you supply an
	// index off the end of the array
	OT_TEST_THROW(setAttribute throw, impl3.setAttribute(iNumAttributes, OT_T("newuri"), OT_T("ot:newqname"), OT_T("newtype"), OT_T("goodbye")), ot::IllegalArgumentException)
	// set the last element to have new values
	OT_TEST_VOID(setAttribute2, impl3.setAttribute(0, OT_T("newuri"), OT_T("ot:newqname"), OT_T("newtype"), OT_T("goodbye")))
	OT_TEST_BOOL(getValue, impl3.getValue(0) == OT_T("goodbye"))

	// test that setQName throws when you supply an index off the end of the array
	OT_TEST_THROW(setQName throw, impl3.setQName(iNumAttributes, OT_T("ot:new")), ot::IllegalArgumentException)

	// test that setQName works
	OT_TEST_VOID(setQName, impl3.setQName(0, OT_T("ot:new")))
	OT_TEST_BOOL(getQName, impl3.getQName(0) == OT_T("ot:new"))

	// test that getIndex (qname) works
	for (i = 0; i < iNumAttributes; i++)
	{
		String qnameStr = String(OT_T("ot:attribute")) + NumUtils::ToString(i);
		OT_TEST_BOOL(getIndex qname, impl1.getIndex(qnameStr) == (int)i)
	}

	// test that getIndex (qname) returns -1 if it cant find the attribute
	OT_TEST_BOOL(getIndex qname returns -1, impl1.getIndex(OT_T("nonexistent")) == -1)
	OT_TEST_BOOL(getIndex, impl3.getIndex(OT_T("nosuchuri"), OT_T("nosuchname")) == -1)

	// test that setType works
	OT_TEST_VOID(setType, impl3.setType(0, OT_T("ID")))
	OT_TEST_BOOL(getType, impl3.getType(0) == OT_T("ID"))
	// check setType throws
	OT_TEST_THROW(setType throw, impl3.setType(iNumAttributes, OT_T("ID")), ot::IllegalArgumentException)

	// test that setURI throws
	OT_TEST_THROW(setURI throw, impl3.setURI(iNumAttributes, OT_T("http://opentop.org/ns")), ot::IllegalArgumentException)
	
	// test that setValue works
	OT_TEST_VOID(setAttributes, impl3.setAttributes(impl2))
	OT_TEST_VOID(setValue, impl3.setValue(0, OT_T("newvalue")))
	OT_TEST_BOOL(getValue, impl3.getValue(0) == OT_T("newvalue"))

	// test that setValue throws
	OT_TEST_THROW(setValue throw, impl3.setValue(iNumAttributes, OT_T("newvalue")), ot::IllegalArgumentException)

OT_END_TESTS(`AttributesImpl')
