//
// ElCel Technology Test Harness 
//
// Class: Features
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/sax/XMLReaderFactory.h"
#include "ot/sax/XMLReader.h"
#include "ot/sax/Locator.h"
#include "ot/sax/SAXFeatures.h"
#include "ot/sax/DefaultHandler.h"
#include "ot/sax/SAXNotSupportedException.h"
#include "ot/net/UnknownHostException.h"
#include "ot/io/StringReader.h"

using namespace ot::sax; 
using namespace ot::net; 
using namespace ot::io; 

class MyFeatureContentHandler : public DefaultHandler
{
	public:
	MyFeatureContentHandler(XMLReader& reader)
		: m_reader(reader)
	{}

	virtual void setDocumentLocator(Locator* pLocator)
	{
		m_rpLocator = pLocator;
	}
	virtual void startDocument()
	{
		m_bIsStandalone = false;
		try
	{
		m_bIsStandalone = m_reader.getFeature(SAXFeatures::is_standalone);
		testFailed(OT_T("get is-standalone"));
	}
	catch(SAXNotSupportedException& e)
	{
		goodCatch(OT_T("get is-standalone"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("get is-standalone"));
	}
	}
	virtual void startElement(const String&, const String&, const String&, const Attributes&)
	{
		try
	{
		m_bIsStandalone = m_reader.getFeature(SAXFeatures::is_standalone); testPassed(OT_T("get is-standalone"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("get is-standalone"));
	}

		//
		// try and set some read-only features
		//
		try
	{
		m_reader.setFeature(SAXFeatures::validation, true);
		testFailed(OT_T("read-only 1"));
	}
	catch(SAXNotSupportedException& e)
	{
		goodCatch(OT_T("read-only 1"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read-only 1"));
	}
		try
	{
		m_reader.setFeature(SAXFeatures::namespaces, true);
		testFailed(OT_T("read-only 2"));
	}
	catch(SAXNotSupportedException& e)
	{
		goodCatch(OT_T("read-only 2"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read-only 2"));
	}
		try
	{
		m_reader.setFeature(SAXFeatures::external_parameter_entities, true);
		testFailed(OT_T("read-only 3"));
	}
	catch(SAXNotSupportedException& e)
	{
		goodCatch(OT_T("read-only 3"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("read-only 3"));
	}
	}
	
	RefPtr<Locator> m_rpLocator;
	XMLReader& m_reader;
	bool m_bIsStandalone;
};


void Features_Tests()
{
	testMessage(OT_T("Starting tests for Features"));

	const String Standalone = OT_T("<?xml version='1.0' standalone='yes'?><test/>");

	RefPtr<XMLReader> rpReader = XMLReaderFactory::CreateXMLReader();
	try
	{
		if(rpReader) {testPassed(OT_T("rpReader"));} else {testFailed(OT_T("rpReader"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("rpReader"));
	}
	RefPtr<MyFeatureContentHandler> rpMyHandler = new MyFeatureContentHandler(*rpReader);

	rpReader->setContentHandler(rpMyHandler.get());

	//
	// test the default value of features
	//
	try
	{
		if(rpReader->getFeature(SAXFeatures::validation) == false) {testPassed(OT_T("default 1"));} else {testFailed(OT_T("default 1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 1"));
	}
	try
	{
		if(rpReader->getFeature(SAXFeatures::external_general_entities) == true) {testPassed(OT_T("default 2"));} else {testFailed(OT_T("default 2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 2"));
	}
	try
	{
		if(rpReader->getFeature(SAXFeatures::external_parameter_entities) == true) {testPassed(OT_T("default 3"));} else {testFailed(OT_T("default 3"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 3"));
	}
	try
	{
		if(rpReader->getFeature(SAXFeatures::lexical_handler_parameter_entities) == false) {testPassed(OT_T("default 4"));} else {testFailed(OT_T("default 4"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 4"));
	}
	try
	{
		if(rpReader->getFeature(SAXFeatures::namespaces) == true) {testPassed(OT_T("default 5"));} else {testFailed(OT_T("default 5"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 5"));
	}
	try
	{
		if(rpReader->getFeature(SAXFeatures::namespace_prefixes) == false) {testPassed(OT_T("default 6"));} else {testFailed(OT_T("default 6"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 6"));
	}
	try
	{
		if(rpReader->getFeature(SAXFeatures::resolve_dtd_uris) == true) {testPassed(OT_T("default 7"));} else {testFailed(OT_T("default 7"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("default 7"));
	}

	RefPtr<StringReader> rpSR = new StringReader(Standalone);
	RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
	try
	{
		rpReader->parse(rpIS.get()); testPassed(OT_T("parse standalone"));
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("parse standalone"));
	}
	try
	{
		if(rpMyHandler->m_bIsStandalone) {testPassed(OT_T("is-standalone"));} else {testFailed(OT_T("is-standalone"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("is-standalone"));
	}


	testMessage(OT_T("End of tests for Features"));
}

