//
// ElCel Technology Test Harness 
//
// Class: Features
//

include(../common.m4)
OT_TEST_INIT

`#include "ot/sax/XMLReaderFactory.h"'
`#include "ot/sax/XMLReader.h"'
`#include "ot/sax/Locator.h"'
`#include "ot/sax/SAXFeatures.h"'
`#include "ot/sax/DefaultHandler.h"'
`#include "ot/sax/SAXNotSupportedException.h"'
`#include "ot/net/UnknownHostException.h"'
`#include "ot/io/StringReader.h"'

using namespace ot::sax; 
using namespace ot::net; 
using namespace ot::io; 

class MyFeatureContentHandler : public DefaultHandler
{
	public:
	MyFeatureContentHandler(XMLReader& reader)
		: m_reader(reader)
	{}

	virtual void setDocumentLocator(Locator* pLocator)
	{
		m_rpLocator = pLocator;
	}
	virtual void startDocument()
	{
		m_bIsStandalone = false;
		OT_TEST_THROW(get is-standalone, m_bIsStandalone = m_reader.getFeature(SAXFeatures::is_standalone), SAXNotSupportedException)
	}
	virtual void startElement(const String&, const String&, const String&, const Attributes&)
	{
		OT_TEST_VOID(get is-standalone, m_bIsStandalone = m_reader.getFeature(SAXFeatures::is_standalone))

		//
		// try and set some read-only features
		//
		OT_TEST_THROW(read-only 1, m_reader.setFeature(SAXFeatures::validation, true), SAXNotSupportedException)
		OT_TEST_THROW(read-only 2, m_reader.setFeature(SAXFeatures::namespaces, true), SAXNotSupportedException)
		OT_TEST_THROW(read-only 3, m_reader.setFeature(SAXFeatures::external_parameter_entities, true), SAXNotSupportedException)
	}
	
	RefPtr<Locator> m_rpLocator;
	XMLReader& m_reader;
	bool m_bIsStandalone;
};

OT_START_TESTS(`Features')

	const String Standalone = OT_T("<?xml version='1.0' standalone='yes'?><test/>");

	RefPtr<XMLReader> rpReader = XMLReaderFactory::CreateXMLReader();
	OT_TEST_BOOL(rpReader, rpReader)
	RefPtr<MyFeatureContentHandler> rpMyHandler = new MyFeatureContentHandler(*rpReader);

	rpReader->setContentHandler(rpMyHandler.get());

	//
	// test the default value of features
	//
	OT_TEST_BOOL(default 1, rpReader->getFeature(SAXFeatures::validation) == false)
	OT_TEST_BOOL(default 2, rpReader->getFeature(SAXFeatures::external_general_entities) == true)
	OT_TEST_BOOL(default 3, rpReader->getFeature(SAXFeatures::external_parameter_entities) == true)
	OT_TEST_BOOL(default 4, rpReader->getFeature(SAXFeatures::lexical_handler_parameter_entities) == false)
	OT_TEST_BOOL(default 5, rpReader->getFeature(SAXFeatures::namespaces) == true)
	OT_TEST_BOOL(default 6, rpReader->getFeature(SAXFeatures::namespace_prefixes) == false)
	OT_TEST_BOOL(default 7, rpReader->getFeature(SAXFeatures::resolve_dtd_uris) == true)

	RefPtr<StringReader> rpSR = new StringReader(Standalone);
	RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
	OT_TEST_VOID(parse standalone, rpReader->parse(rpIS.get()))
	OT_TEST_BOOL(is-standalone, rpMyHandler->m_bIsStandalone)

OT_END_TESTS(`Features')

