//
// ElCel Technology Test Harness 
//
// Class: Locator
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/sax/XMLReaderFactory.h"
#include "ot/sax/XMLReader.h"
#include "ot/sax/Locator.h"
#include "ot/sax/DefaultHandler.h"
#include "ot/net/UnknownHostException.h"

using namespace ot::sax; 
using namespace ot::net; 

String s1 = OT_T("http://a.b.c/x.xml");

class MyContentHandler : public DefaultHandler
{
	virtual void setDocumentLocator(Locator* pLocator)
	{
		m_rpLocator = pLocator;
	}
	virtual void startDocument()
	{
		try
	{
		if(m_rpLocator->getLineNumber() == -1) {testPassed(OT_T("test Line"));} else {testFailed(OT_T("test Line"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("test Line"));
	}
		try
	{
		if(m_rpLocator->getColumnNumber() == -1) {testPassed(OT_T("test Column"));} else {testFailed(OT_T("test Column"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("test Column"));
	}
		try
	{
		if(m_rpLocator->getSystemId() == s1) {testPassed(OT_T("test systemId"));} else {testFailed(OT_T("test systemId"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("test systemId"));
	}
	}
	RefPtr<Locator> m_rpLocator;
};


void Locator_Tests()
{
	testMessage(OT_T("Starting tests for Locator"));


	RefPtr<XMLReader> rpReader = XMLReaderFactory::CreateXMLReader();
	try
	{
		if(rpReader) {testPassed(OT_T("rpReader"));} else {testFailed(OT_T("rpReader"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("rpReader"));
	}
	RefPtr<MyContentHandler> rpMyHandler = new MyContentHandler;

	rpReader->setContentHandler(rpMyHandler.get());
	try
	{
		if(rpReader->getContentHandler() == rpMyHandler) {testPassed(OT_T("check content handler"));} else {testFailed(OT_T("check content handler"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("check content handler"));
	}

	try
	{
		rpReader->parse(s1);
		testFailed(OT_T("unknown host"));
	}
	catch(UnknownHostException& e)
	{
		goodCatch(OT_T("unknown host"), e.toString());
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("unknown host"));
	}


	testMessage(OT_T("End of tests for Locator"));
}

