//
// ElCel Technology Test Harness 
//
// Class: XMLReader
//

include(../common.m4)
OT_TEST_INIT

`#include "ot/sax/XMLReaderFactory.h"'
`#include "ot/sax/XMLReader.h"'
`#include "ot/sax/Locator.h"'
`#include "ot/sax/SAXFeatures.h"'
`#include "ot/sax/DefaultHandler.h"'
`#include "ot/sax/SAXNotSupportedException.h"'
`#include "ot/sax/SAXParseException.h"'
`#include "ot/net/UnknownHostException.h"'
`#include "ot/io/StringReader.h"'

using namespace ot::sax; 
using namespace ot::net; 
using namespace ot::io; 

size_t NamespaceAttributes = 0;

class MyXMLReaderHandler : public DefaultHandler
{
public:

	virtual void setDocumentLocator(Locator* pLocator)
	{
		m_rpLocator = pLocator;
	}
	virtual void startDocument()
	{
	}
	virtual void startElement(const String& ns, const String& local, const String& qname, const Attributes& atts)
	{
		OT_TEST_BOOL(check ns uri, ns == OT_T("http://www.opentop.org/ns"))
		OT_TEST_BOOL(check local, local == OT_T("test"))
		OT_TEST_BOOL(check qname, qname == OT_T("ot:test"))
		OT_TEST_BOOL(check atts, atts.getLength() == (1 + NamespaceAttributes))
		OT_TEST_BOOL(check att type, atts.getType(0) == OT_T("CDATA"))
	}
	virtual void error(const SAXParseException& e)
	{
		throw e;
	}	

	RefPtr<Locator> m_rpLocator;
	bool m_bIsStandalone;
};

OT_START_TESTS(`XMLReader')

	const String dtd   = OT_T("<!DOCTYPE test [")
	                     OT_T("<!ELEMENT test (#PCDATA)*>")
	                     OT_T("<!ATTLIST test attr1 CDATA #REQUIRED attr2 (yes|no) 'yes'>")
	                     OT_T("]>");
	const String doc   = OT_T("<ot:test xmlns:ot='http://www.opentop.org/ns' attr1='attr1'/>");
	const String valid = dtd + doc;

	RefPtr<XMLReader> rpReader = XMLReaderFactory::CreateXMLReader();
	RefPtr<MyXMLReaderHandler> rpHandler = new MyXMLReaderHandler();

	//
	// Set the XMLreader into a validating parser
	//
	OT_TEST_VOID(set validation, rpReader->setFeature(SAXFeatures::validation, true))

	//
	// Register a content handler, which tests correct namespace processing
	//
	rpReader->setContentHandler(rpHandler.get());
	{
		RefPtr<StringReader> rpSR = new StringReader(dtd);
		RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
		OT_TEST_THROW(parse notwf, rpReader->parse(rpIS.get()), SAXParseException)
	}
	{
		RefPtr<StringReader> rpSR = new StringReader(doc);
		RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
		OT_TEST_VOID(parse invalid, rpReader->parse(rpIS.get()))
	}
	{
		RefPtr<StringReader> rpSR = new StringReader(valid);
		RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
		OT_TEST_VOID(parse valid, rpReader->parse(rpIS.get()))
	}

	//
	// Now let's try that again, this time with namespace prefix reporting switched on
	//
	OT_TEST_VOID(switch on prefix reporting, rpReader->setFeature(SAXFeatures::namespace_prefixes, true))
	NamespaceAttributes = 1;
	{
		RefPtr<StringReader> rpSR = new StringReader(valid);
		RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
		OT_TEST_VOID(parse ns-prefix, rpReader->parse(rpIS.get()))
	}

	//
	// Register an error handler, which throws a SAXParseException on validity errors
	//
	rpReader->setErrorHandler(rpHandler.get());
	{
		RefPtr<StringReader> rpSR = new StringReader(doc);
		RefPtr<InputSource> rpIS = new InputSource(rpSR.get());
		OT_TEST_THROW(parse invalid 2, rpReader->parse(rpIS.get()), SAXParseException)
	}

OT_END_TESTS(`XMLReader')

