//
// ElCel Technology Test Harness 
//
// Class: Base64
//

include(../common.m4)
OT_TEST_INIT

`#include "ot/base/Character.h"'
`#include "ot/base/ArrayAutoPtr.h"'
`#include "ot/util/Base64.h"'
`#include <string.h>'

using namespace ot::util; 

void testBase64(const char* pPlain, const char* pBase64)
{
	//
	// First convert the plain test to base-64
	//
	const size_t encSize = Base64::GetEncodedLength((const Byte*)pPlain, (const Byte*)pPlain+strlen(pPlain));
	ArrayAutoPtr<Byte> apBuffer(new Byte[encSize]);
	Byte* pNext;
	OT_TEST_BOOL(Encode, Base64::Encode((const Byte*)pPlain, (const Byte*)pPlain+strlen(pPlain), apBuffer.get(), apBuffer.get()+encSize, pNext) == Base64::ok)
	size_t actualLength = pNext - apBuffer.get();
	OT_TEST_BOOL(length test, encSize == actualLength)
	OT_TEST_BOOL(Base64 match, strncmp((const char*)apBuffer.get(), pBase64, encSize) == 0)
	//
	// And then convert back again
	//
	const size_t maxDecodedSize = Base64::GetMaxDecodedLength((const Byte*)pBase64, (const Byte*)pBase64+strlen(pBase64));
	OT_TEST_BOOL(decoded length, maxDecodedSize < encSize)

	OT_TEST_BOOL(Decode, Base64::Decode((const Byte*)pBase64, (const Byte*)pBase64+strlen(pBase64), apBuffer.get(), apBuffer.get()+encSize, pNext) == Base64::ok)
	actualLength = pNext - apBuffer.get();
	OT_TEST_BOOL(dec length test, maxDecodedSize >= actualLength)
	OT_TEST_BOOL(plain match, strncmp((const char*)apBuffer.get(), pPlain, actualLength) == 0)
}

OT_START_TESTS(`Base64')

    //
	const char* plain_1 = "Hello World!\r\n";
	const char* b64_1   = "SGVsbG8gV29ybGQhDQo=";
	const char* plain_2 = "qwertyuiop1234567890bvcxz\r\n";
	const char* b64_2   = "cXdlcnR5dWlvcDEyMzQ1Njc4OTBidmN4eg0K";
	const char* plain_3 = "Q";
	const char* b64_3   = "UQ==";
	const char* plain_4 = "1234567890.<ZXC:@\r\n";
	const char* b64_4   = "MTIzNDU2Nzg5MC48WlhDOkANCg==";
	const char* plain_5 = "blat!\r\n";
	const char* b64_5   = "YmxhdCENCg==";

	testBase64(plain_1, b64_1);
	testBase64(plain_2, b64_2);
	testBase64(plain_3, b64_3);
	testBase64(plain_4, b64_4);
	testBase64(plain_5, b64_5);

OT_END_TESTS(`Base64')

