//
// ElCel Technology Test Harness 
//
// Class: StringIterator
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/base/Character.h"
#include "ot/base/String.h"
#include "ot/base/StringIterator.h"

#include <algorithm>


void StringIterator_Tests()
{
	testMessage(OT_T("Starting tests for StringIterator"));

    //

	String seq = OT_T("Hello \t World!\n");
	String hello  = OT_T("Hello");
	String world  = OT_T("World!");
	String ws = OT_T(" \t\n");

    Character nbsp(0xa0);
	String ws2 = ws + nbsp.toString();
	String seq2 = ws + hello + nbsp.toString() + world + ws;

	StringIterator begin(seq.data());
	StringIterator end(seq.data()+seq.size());
	StringIterator firstDelim(ws.data());
	StringIterator endDelim(ws.data()+ws.size());

	//
	// Test various construction methods
	//
	StringIterator space;
	StringIterator space2 = space;
	StringIterator space3(space2);
	try
	{
		if(space == space2 && space2 == space3) {testPassed(OT_T("equality"));} else {testFailed(OT_T("equality"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("equality"));
	};

	space3 = std::find_first_of(begin, end, firstDelim, endDelim);
	try
	{
		if(space3==(seq.data()+5)) {testPassed(OT_T("find_first_of"));} else {testFailed(OT_T("find_first_of"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("find_first_of"));
	};

	StringIterator iW = std::find(begin, end, OT_T('W'));
	try
	{
		if(*iW == OT_T('W')) {testPassed(OT_T("find"));} else {testFailed(OT_T("find"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("find"));
	};


	testMessage(OT_T("End of tests for StringIterator"));
}

