//
// ElCel Technology Test Harness 
//
// Class: StringTokenizer
//




#include "ot/base/System.h"
#include "ot/io/Console.h"
#include "ot/base/Exception.h"
#include "ot/base/StringUtils.h"
#include "ot/util/AttributeListParser.h"

using namespace ot;
using namespace ot::io;
using namespace ot::util;

String getTestAttribute(const String& name);
void testMessage(const String& msg);
void testFailed(const String& test);
void testPassed(const String& test);
void goodCatch(const String& test, const String& eMsg);
void uncaughtException(const String& e, const String& test);


#include "ot/base/Character.h"
#include "ot/util/StringTokenizer.h"

using namespace ot::util; 


void StringTokenizer_Tests()
{
	testMessage(OT_T("Starting tests for StringTokenizer"));

    //

	String seq = OT_T("Hello \t World!\n");
	String hello  = OT_T("Hello");
	String world  = OT_T("World!");
	String ws = OT_T(" \t\n");

    Character nbsp(0xa0);
	String ws2 = ws + nbsp.toString();
	String seq2 = ws + hello + nbsp.toString() + world + ws;
	String seq3 = OT_T("  H\t \nW ");

	std::vector<String> vec1 = StringTokenizer(seq).toVector();
	try
	{
		if(vec1.size()==2) {testPassed(OT_T("tok0"));} else {testFailed(OT_T("tok0"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok0"));
	};
	try
	{
		if(vec1[0]==hello) {testPassed(OT_T("tok1"));} else {testFailed(OT_T("tok1"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok1"));
	};
	try
	{
		if(vec1[1]==world) {testPassed(OT_T("tok2"));} else {testFailed(OT_T("tok2"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok2"));
	};

	std::vector<String> vec2 = StringTokenizer(seq2).toVector();
	try
	{
		if(vec2.size()==1) {testPassed(OT_T("tok3"));} else {testFailed(OT_T("tok3"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok3"));
	};

	std::vector<String> vec3 = StringTokenizer(seq2, ws2, true).toVector();
	try
	{
		if(vec3.size()==5) {testPassed(OT_T("tok4"));} else {testFailed(OT_T("tok4"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok4"));
	};
	try
	{
		if(vec3[2]==nbsp.toString()) {testPassed(OT_T("tok5"));} else {testFailed(OT_T("tok5"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok5"));
	};

	std::vector<String> vec4 = StringTokenizer(seq3, ws, true, false).toVector();
	try
	{
		if(vec4.size()==seq3.size()) {testPassed(OT_T("tok6"));} else {testFailed(OT_T("tok6"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok6"));
	};
	try
	{
		if(vec4[0]==OT_T(" ")) {testPassed(OT_T("tok7"));} else {testFailed(OT_T("tok7"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok7"));
	};
	try
	{
		if(vec4[2]==OT_T("H")) {testPassed(OT_T("tok8"));} else {testFailed(OT_T("tok8"));}
	}
	catch(Exception& e)
	{
		uncaughtException(e.toString(), OT_T("tok8"));
	};


	testMessage(OT_T("End of tests for StringTokenizer"));
}

