//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:49 $
//
//==============================================================================

#include "BasicOption.h"
#include "messages.h"

#include "ot/base/System.h"
#include "ot/util/MessageFormatter.h"

OT_AUXIL_NAMESPACE_BEGIN 

using namespace util;

const String sAuxil = OT_T("auxil");

BasicOption::BasicOption(const String& longName, CharType shortName, ArgumentType eArgumentType) :
	m_bPresent(false),
	m_eArgumentType(eArgumentType),
	m_longName(longName),
	m_shortName(shortName)
{
}

BasicOption::~BasicOption()
{
}

bool BasicOption::testShortOption(CharType option) const
{
	return (option == m_shortName);
}

bool BasicOption::testLongOption(const String& option) const
{
	// missing from gnu libstdc++
	// return m_longName.compare(0, option.size(), option)==0;
	return m_longName.substr(0, option.size()) == option;
}

BasicOption::ArgumentType BasicOption::getArgumentType() const
{
	return m_eArgumentType;
}

void BasicOption::setArgument(const String& arg)
{
	m_argument = arg;
}

const String BasicOption::getArgument() const
{
	return m_argument;
}

const String& BasicOption::getLongName() const
{
	return m_longName;
}

CharType BasicOption::getShortName() const
{
	return m_shortName;
}

//==============================================================================
// BasicOption::setPresent
//
/**
   Called by the CommandLineParser to indicate to the option that is has been
   specified on the command line.
   
   @param option how the option was specified on the command line
   @param optionTypeUsed indicates whether the long or short option was used

   @throws CommandLineException if the option is a duplicate
*/
//==============================================================================
void BasicOption::setPresent(const String& option, OptionType /*optionTypeUsed*/)
{
	if(m_bPresent)
	{
		const String& errMsg = MessageFormatter::Format(
			System::GetSysMessage(sAuxil, EAUXIL_OPTIONDUPLICATE,
			"duplicate option: {0}"),
			option);

		throw CommandLineException(option, errMsg);
	}

	m_bPresent = true;
}

bool BasicOption::isPresent() const
{
	return m_bPresent;
}

OT_AUXIL_NAMESPACE_END 
