//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:50 $
//
//==============================================================================

#include "BooleanOption.h"
#include "messages.h"

#include "ot/base/IllegalArgumentException.h"
#include "ot/base/System.h"
#include "ot/util/MessageFormatter.h"

OT_AUXIL_NAMESPACE_BEGIN

const String sAuxil = OT_T("auxil");

using namespace util;

//==============================================================================
// BooleanOption::BooleanOption
//
/**
   Constructs a BooleanOption.

   @param longName the long name of this option
   @param shortName the short name of this option.  Specify 0 to indicate there
          is no short option name
   @param bDefault default value to report if the option is not specified on the
          command line
   @throws IllegalArgumentException if @c longName starts with '--'
*/
//==============================================================================
BooleanOption::BooleanOption(const String& longName, CharType shortName, bool bDefault) :
	BasicOption(longName, shortName, none),
	m_bValue(bDefault)
{
	// compare() params wrong way round in gnu libstdc++
	// if(longName.compare(0, 2, OT_T("--"))==0)
	if(longName.substr(0, 2) == OT_T("--"))
	{
		const String& errMsg = MessageFormatter::Format(
			System::GetSysMessage(sAuxil, EAUXIL_OPTIONNAMEBOOL,
			"illegal boolean option name: {0}"),
			longName);

		throw IllegalArgumentException(errMsg);
	}
}

const String BooleanOption::getArgument() const
{
	if(m_bValue)
		return OT_T("true");
	else
		return OT_T("false");
}

	
void BooleanOption::setPresent(const String& howSpecified, OptionType optionTypeUsed)
{
	BasicOption::setPresent(howSpecified, optionTypeUsed);

	if(optionTypeUsed == ShortOption)
	{
		// when short option is used for boolean options we reverse the value
		m_bValue = !m_bValue;
	}
	else //long option specified
	{
		if(howSpecified.substr(0, 2) == OT_T("no"))
			m_bValue = false;
		else
			m_bValue = true;
	}
}

bool BooleanOption::testLongOption(const String& option) const
{
	if(option.substr(0, 2) == OT_T("no"))
	{
		String comparitor = option.substr(2);
		return (getLongName().substr(0, comparitor.size()) == comparitor);
	}
	else
	{
		return (getLongName().substr(0, option.size()) == option);
	}
}

bool BooleanOption::getValue() const
{
	return m_bValue;
}

OT_AUXIL_NAMESPACE_END

