//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:51 $
//
//==============================================================================
//
// Class: MessageSet
// 
// Overview
// --------
// An implementation of MessageFactory that loads messages from a
// file.
//
//==============================================================================

#include "MessageSet.h"

OT_AUXIL_NAMESPACE_BEGIN

MessageSet::MessageSet(const String& org, const String& app) :
	m_org(org),
	m_app(app)
{
}

void MessageSet::addMessage(size_t messageID, const String& message)
{
	m_messageMap[messageID] = message;
}

bool MessageSet::getMessageText(size_t messageID, String& ret) const
{
	MessageMap::const_iterator iter = m_messageMap.find(messageID);
	if(iter != m_messageMap.end())
	{
		ret = (*iter).second;
		return true;
	}
	else
	{
		return false;
	}
}

const String& MessageSet::getOrganizationName() const
{
	return m_org;
}

const String& MessageSet::getApplicationName() const
{
	return m_app;
}

OT_AUXIL_NAMESPACE_END
