//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:51 $
//
//==============================================================================

#include "NetAccessHelper.h"
#include "BasicOption.h"

#include "ot/base/System.h"
#include "ot/base/NumUtils.h"
#include "ot/net/HttpURLConnection.h"
#include "ot/net/FtpURLConnection.h"

OT_AUXIL_NAMESPACE_BEGIN

using namespace net;

NetAccessHelper::NetAccessHelper(const String& HTTPProxy,
                                 const String& proxyUser,
                                 const String& proxyPassword,
                                 const String& user,
                                 const String& password)
{
	//
	// If a HTTP proxy server has been provided, then use it
	//
	String sHttpProxy = HTTPProxy.empty() 
	                  ? System::GetEnvironmentString(OT_T("ET_HTTP_PROXY"))
					  : HTTPProxy;

	if(!sHttpProxy.empty())
	{
		System::SetPropertyBool(OT_T("http.proxySet"), true);
		size_t portPos = sHttpProxy.find(OT_T(":"));
		if(portPos != String::npos)
		{
			String host = sHttpProxy.substr(0, portPos);
			String port = sHttpProxy.substr(portPos+1);
			System::SetProperty(OT_T("http.proxyHost"), host);
			System::SetPropertyLong(OT_T("http.proxyPort"), NumUtils::ToLong(port));
		}
		else
		{
			System::SetProperty(OT_T("http.proxyHost"), sHttpProxy);
		}
	}

	//
	// If a proxy password has been provided, then use it
	//
	m_proxyPassword = proxyPassword.empty()
	                ? System::GetEnvironmentString(OT_T("ET_HTTP_PROXY_PASSWORD"))
	                : proxyPassword;


	//
	// If a proxy user has been provided, then use it
	//
	m_proxyUser = proxyUser.empty()
		        ? System::GetEnvironmentString(OT_T("ET_HTTP_PROXY_USER"))
	            : proxyUser;

	//
	// If a password has been provided, then use it
	//
	m_password = password.empty()
		       ? System::GetEnvironmentString(OT_T("ET_HTTP_PASSWORD"))
	           : password;

	//
	// If a user has been provided, then use it
	//
	m_user = user.empty()
		   ? System::GetEnvironmentString(OT_T("ET_HTTP_USER"))
	       : user;
}

bool NetAccessHelper::getPasswordAuthentication(InetAddress* /*pAddr*/, int /*port*/,
	const String& /*protocol*/, const String& /*prompt*/, const String& /*scheme*/,
	bool bProxy, String& userRet, String& passwordRet)
{
	if(bProxy && !m_proxyUser.empty())
	{
		userRet = m_proxyUser;
		passwordRet = m_proxyPassword;
	}
	else
	{
		userRet = m_user;
		passwordRet = m_password;
	}
	return true;
}

OT_AUXIL_NAMESPACE_END
