//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:39 $
//
//==============================================================================
// Class: OSException
// 
/**
	@class ot::OSException
	
	@brief Thrown when @OpenTop makes a system call which results
	in an unexpected return code.
*/
//==============================================================================

#include "OSException.h"
#include "SystemUtils.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// OSException::OSException
//
/**
   Protected default constructor.
*/
//==============================================================================
OSException::OSException()
{
}

//==============================================================================
// OSException::OSException
//
/**
   Constructs a OSException with a system error code which is used to
   retrieve an operating system error message.
   @param errCode the system error code that gave rise to the exception
*/
//==============================================================================
OSException::OSException(long errCode) :
	RuntimeException(SystemUtils::GetSystemErrorString(errCode))
{
}

//==============================================================================
// OSException::OSException
//
/**
   Constructs a OSException with a system error code and additional message.
   The error code is used to retrieve an operating system error message.
   @param errCode the system error code that gave rise to the exception
   @param message further detail
*/
//==============================================================================
OSException::OSException(long errCode, const String& message)
{
	String errMsg;
	if(!message.empty())
	{
		 errMsg = message;
		 errMsg += OT_T(": ");
	}
	errMsg += SystemUtils::GetSystemErrorString(errCode);
	setMessage(errMsg);
}

//==============================================================================
// OSException::getErrorCode
//
/**
   Returns the system error code that gave rise to the exception.
*/
//==============================================================================
long OSException::getErrorCode() const
{
	return m_errCode;
}

OT_BASE_NAMESPACE_END
