//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:24:45 $
//
//==============================================================================
//
// Class: SystemMonitor
// 
/**
	@class ot::SystemMonitor
	
	@brief Application helper class to terminate the @OpenTop library cleanly.

    Normally instantiated in the @c main() function, SystemMonitor uses its destructor
    to ensure a clean shutdown of the application by calling System::Terminate().

    @code
    #include "ot/base/SystemMonitor.h"
    using namespace ot;

    int main(int argc, char* argv[])
    {
        // create a SystemMonitor instance to ensure clean termination
        SystemMonitor monitor;
        
        // do anything here...

        return (0);
    }
    @endcode

	@sa System::Terminate()
*/
//==============================================================================

#include "SystemMonitor.h"
#include "System.h"
#include "Thread.h"

OT_BASE_NAMESPACE_BEGIN

//==============================================================================
// SystemMonitor::~SystemMonitor
//
/**
   Destructor.

   Performs any essential termination of the @OpenTop library, including
   calling the System::Terminate() function.
*/
//==============================================================================
SystemMonitor::~SystemMonitor()
{
	try
	{
		System::Terminate();
	}
	catch(...)
	{
	}
}

OT_BASE_NAMESPACE_END
