//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:06 $
//
//==============================================================================

#include "UTF8codecvt.h"

OT_CVT_NAMESPACE_BEGIN

//==============================================================================
// UTF8codecvt::do_in
//
// Decode a sequence of external characters into a sequence of internal
// characters.  As this is a job required elsewhere (by programs not using
// IOStreams) we delegate to another class that knows how to do it.
//==============================================================================
UTF8codecvt::result UTF8codecvt::do_in(stateT& state,
	const externT *from, const externT *from_end, const externT *& from_next,
	internT *to, internT *to_limit, internT *& to_next) const
{
	switch(const_cast<UTF8Converter&>(m_cvt).decode(from, from_end, from_next,
	                  to, to_limit, to_next))
	{
	case UTF8Converter::outputExhausted:
	case UTF8Converter::inputExhausted:
		return partial;
	case UTF8Converter::ok:
		return ok;
	default:
		return error;
	}
}

//==============================================================================
// UTF8codecvt::do_out
//
// Encode a sequence of internal characters into a sequence of external
// characters.  As this is a job required elsewhere (by programs not using
// IOStreams) we delegate to another class that knows how to do it.
//==============================================================================
UTF8codecvt::result UTF8codecvt::do_out(stateT& state,
		const internT *from, const internT *from_end, const internT *& from_next,
		externT *to, externT *to_limit, externT *& to_next) const
{
	switch(const_cast<UTF8Converter&>(m_cvt).encode(from, from_end, from_next,
	                             to, to_limit, to_next))
	{
	case UTF8Converter::outputExhausted:
	case UTF8Converter::inputExhausted:
		return partial;
	case UTF8Converter::ok:
		return ok;
	default:
		return error;
	}
}

//==============================================================================
// UTF8codecvt::do_unshift
//
//==============================================================================
UTF8codecvt::result UTF8codecvt::do_unshift(stateT& state,
	externT *to, externT *to_limit, externT *& to_next) const
{
	return (ok);
}

//==============================================================================
// UTF8codecvt::do_length
//
//==============================================================================
int UTF8codecvt::do_length(stateT& state, const externT *from,
	const externT *from_end, size_t max) const 
{
	size_t len = m_cvt.getDecodedLength(from, from_end);
	return len > max ? max : len;
}

//==============================================================================
// UTF8codecvt::do_always_noconv
//
// Return true if this is a non-converting codecvt.
//==============================================================================
bool UTF8codecvt::do_always_noconv() const
{
	return (false); // false = we do convert!
}

//==============================================================================
// UTF8codecvt::do_max_length
//
// Return the maximum number of external characters required to encode a 
// single internal character.
//==============================================================================
int UTF8codecvt::do_max_length() const 
{
	return m_cvt.getMaxEncodedLength();
}

//==============================================================================
// UTF8codecvt::do_encoding
//
// Return -1 if the encoding sequence is state dependent.  Otherwise return the
// constant number of external characters that are needed to produce one
// internal character, or 0 if this number is not constant
//==============================================================================
int UTF8codecvt::do_encoding() const 
{
	return (0); // variable number of bytes reqd to encode a single character
}

OT_CVT_NAMESPACE_END

