//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:15 $
//
//==============================================================================
//
// Class: FilterOutputStream
/**
	@class ot::io::FilterOutputStream
	
	@brief A base class that @a derives from OutputStream but also
	@a contains another OutputStream which is used as the output sink.
	
    FilterOutputStream and its sister class FilterInputStream provide
	an extensible framework for building chains of processing.

    The FilterOutputStream class overrides all methods of OutputStream with 
	versions that pass requests to the contained output stream.
	Derived classes of FilterOutputStream are expected to further override
	some of these methods to perform some useful function before the data
	is written out to the sink (which may be yet another FilterOutputStream).
*/
//==============================================================================

#include "FilterOutputStream.h"

#include "ot/base/NullPointerException.h"

OT_IO_NAMESPACE_BEGIN

//==============================================================================
// FilterOutputStream::FilterOutputStream
//
/**
   Constructs a FilterOutputStream using @c pOutputStream as the contained
   output stream.

   @throws NullPointerException if @c pOutputStream is null.
*/
//==============================================================================
FilterOutputStream::FilterOutputStream(OutputStream* pOutputStream) :
	m_rpOutputStream(pOutputStream)
{
	if(!pOutputStream) throw NullPointerException();
}

void FilterOutputStream::close()
{
	m_rpOutputStream->close();
}

void FilterOutputStream::flush()
{
	m_rpOutputStream->flush();
}

void FilterOutputStream::flushBuffers()
{
	m_rpOutputStream->flushBuffers();
}

void FilterOutputStream::write(Byte x)
{
	m_rpOutputStream->write(x);
}

void FilterOutputStream::write(const Byte* pBuffer, size_t bufLen)
{
	m_rpOutputStream->write(pBuffer, bufLen);
}

//==============================================================================
// FilterOutputStream::getOutputStream
//
/**
   Returns the contained output stream.
*/
//==============================================================================
RefPtr<OutputStream> FilterOutputStream::getOutputStream() const
{
	return m_rpOutputStream;
}

OT_IO_NAMESPACE_END
