//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.2 $
// $Date: 2003/08/28 22:48:46 $
//
//==============================================================================
// Class: MalformedInputException
// 
/**
	@class ot::io::MalformedInputException
	
	@brief Thrown when an InputStreamReader, that has been configured to abort
	when it encounters encoding errors, reads a malformed byte sequence.

	@sa CodeConverter
*/
//==============================================================================

#include "MalformedInputException.h"

#include "ot/base/ArrayAutoPtr.h"
#include "ot/base/StringUtils.h"

#include <stdio.h>

OT_IO_NAMESPACE_BEGIN

//==============================================================================
// MalformedInputException::MalformedInputException
//
/**
   Constructs a MalformedInputException with a detail message and
   a decoder.

   @param message the detail message
   @param pDecoder the CodeConverter giving rise to the exception
*/
//==============================================================================
MalformedInputException::MalformedInputException(const String& message, CodeConverter* pDecoder) : 
	CharacterCodingException(message, pDecoder)
{
}

//==============================================================================
// MalformedInputException::MalformedInputException
//
/**
	Constructs a MalformedInputException with a Byte array and a decoder.
	@param pStart pointer to the start of the array of badly encoded bytes
	@param len length of the array
	@param pDecoder the CodeConverter giving rise to the exception
*/
//==============================================================================
MalformedInputException::MalformedInputException(const Byte* pStart, size_t len, CodeConverter* pConv) :
	CharacterCodingException(pConv)
{
	OT_DBG_ASSERT(pStart!=0 && len!=0);

	size_t bufLen = len * 2 + 1;
	ArrayAutoPtr<char> apBuffer(new char[bufLen]);
	size_t j=0;
	for(size_t i=0; i<len && j<bufLen-2; i++)
	{
		j += ::sprintf(apBuffer.get()+j, "%02X", (unsigned)*(pStart+i));
	}

	String errMsg = OT_T("invalid ");
	errMsg += pConv->getEncodingName();
	errMsg += OT_T(" sequence: 0x");
	errMsg += StringUtils::FromLatin1(apBuffer.get());
	
	setMessage(errMsg);
}

//==============================================================================
// MalformedInputException::getExceptionType
//
//==============================================================================
String MalformedInputException::getExceptionType() const
{
	return OT_T("MalformedInputException");
}

OT_IO_NAMESPACE_END
