//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:31 $
//
//==============================================================================

#include "NvtAsciiInputStream.h"
#include "ProtocolException.h"

#include "ot/base/SystemUtils.h"

OT_NET_NAMESPACE_BEGIN

#if defined(WIN32)
	const char* EOLBytes = "\r\n";
	const size_t EOLBytesLength = 2;
#else
	const char* EOLBytes = "\n";
	const size_t EOLBytesLength = 1;
#endif

NvtAsciiInputStream::NvtAsciiInputStream(InputStream* pInputStream) :
	FilterInputStream(pInputStream)
{
}

//==============================================================================
// NvtAsciiInputStream::read
//
// Translate <CRLF> to local line-feed convention.
//==============================================================================
long NvtAsciiInputStream::read(Byte* pBuffer, size_t bufLen)
{
	long bytesRead = FilterInputStream::read(pBuffer, bufLen);

#if defined(WIN32)

	return bytesRead;

#else

	// In the UNIX domain, the local convention is <LF>
	//
	// translate all <CRLF> pairs to standalone <LF>

	if(bytesRead == EndOfFile)
	{
		return EndOfFile;
	}

	Byte* pNext = pBuffer;
	Byte* pEnd = pBuffer + bytesRead;
	const Byte CR = 13;
	const Byte LF = 10;

	while(pNext < pEnd)
	{
		const Byte b = *pNext++;
		if(b == CR)
		{
			//
			// Reset the <CR> to <LF> and check to see if we have enough
			// buffer space to zap the next byte
			//
			*--pNext = LF;
			if(pNext >= pEnd)
			{
				int b = FilterInputStream::read(); // eat the next byte - it must be <null> of <LF>
				if(b != 0 && b != LF)
				{
					throw ProtocolException(OT_T("invalid NVT-ASCII byte sequence"));
				}
			}
			else
			{
				//
				// We can now deal with the second half of the <CRLF> pair
				// which simply needs to be discarded
				//
				::memmove(pNext, pNext+1, pEnd-pNext-1);
				--bytesRead;
				--pEnd;
			}
		}
	}

	return bytesRead;

#endif // WIN32
}

OT_NET_NAMESPACE_END
