//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:31 $
//
//==============================================================================

#include "NvtAsciiOutputStream.h"

#include "ot/base/NullPointerException.h"

OT_NET_NAMESPACE_BEGIN

NvtAsciiOutputStream::NvtAsciiOutputStream(OutputStream* pOutputStream) :
	FilterOutputStream(pOutputStream),
	m_bCRSeen(false)
{
}

//==============================================================================
// NvtAsciiOutputStream::write
//
//==============================================================================
void NvtAsciiOutputStream::write(const Byte* pBuffer, size_t bufLen)
{
	if(!pBuffer) throw NullPointerException();

	//
	// Search for <CR> bytes not followed by a <LF> and replace by <CRLF>
	//
	const Byte* pEnd = pBuffer+bufLen;
	const Byte* pNext = pBuffer;
	const Byte* pLast = pBuffer;
	const Byte CR = 13;

	while(pNext < pEnd)
	{
		const Byte b = *pNext;
		if(b == CR)
		{
			m_bCRSeen = true;
		}
		else if(b == 10)
		{
			if(!m_bCRSeen)
			{
				// flush out what's gone so far
				FilterOutputStream::write(pLast, pNext-pLast);
				pLast = pNext;
				// write an additional <CR>
				FilterOutputStream::write(&CR, 1);
			}
			m_bCRSeen = false;
		}
		++pNext;
	}
	OT_DBG_ASSERT(pEnd == pNext);
	FilterOutputStream::write(pLast, pNext-pLast);
}

OT_NET_NAMESPACE_END
