#
# Makefile for the OpenTop library
#
# This makefile can build any of the multiple configurations.
#
# See the file "makeinclude" for details.
#
#  THREAD=[single|multi]
#  MODE=[release|debug]
#  CHAR=[char|wchar]
#  TYPE=[shared|static]
#

#
# The base-name of the library
#
BASE_NAME=opentop

#
# SRC_DIRS is a local macro that contains a list of the sub-directories
# containing source files
#
SRC_DIRS = base io cvt net auxil util

#
# Each subdirectory contains a "package.mk" file containing the
# object files contained therein
#
include $(addsuffix /package.mk, ${SRC_DIRS})

#
# OBJS contains the names of all the bare files comprising
# the library (without a suffix)
#
OBJS = ${AUXIL_OBJS}\
       ${BASE_OBJS}\
       ${CVT_OBJS}\
       ${IO_OBJS}\
       ${NET_OBJS}\
       ${UTIL_OBJS}

#
# When compiling the opentop library, the symbol OT_OPENTOP_EXPORT must be defined.
# Note that it should not be defined when compiling programs that *use* the library.
#
CXXFLAGS = -DOT_IOS_EXPORT

#
# The OpenTop include directory can be specified in absolute or relative
# terms.  
#
OPENTOP_INC_DIR=../

#
# DESTINATION_DIR is the directory that the library is built into.
# This is not the same place where the object files are created.
#
DESTINATION_DIR=../lib

MAKE_FILE=-f opentop.mak
BUILD_TOOLS=../buildtools
include ${BUILD_TOOLS}/makeinclude
