#
# Makefile for the OpenTop library
#
# This makefile can build any of the multiple configurations.
#
# See the file "makeinclude" for details.
#
#  THREAD=[single|multi]
#  MODE=[release|debug]
#  CHAR=[char|wchar]
#  TYPE=[shared|static]
#

#
# The base-name of the library
#
BASE_NAME=otxml

#
# SRC_DIRS is a global macro that contains a list of the sub-directories
# containing source files.  The VPATH (view path) macro is set in
# dependent makefiles accordingly
#
SRC_DIRS = xml xmlcat sax

#
# Each subdirectory contains a "package.mk" file containing the
# object files contained therein
#
include $(addsuffix /package.mk, ${SRC_DIRS})

#
# OBJS contains the names of all the bare files comprising
# the library (without a suffix)
#
OBJS = ${XML_OBJS}\
       ${XMLCAT_OBJS}\
       ${SAX_OBJS}

#
# When compiling the otxml library, the symbol OT_OTXML_EXPORT must be defined.
# Note that it should not be defined when compiling programs that *use* the library.
#
CXXFLAGS = -DOT_XML_EXPORT

#
# When creating a DLL under Windows we need to import the opentop library
#
OT_LIBS = opentop
PROJ_LIB_DIRS = ../lib

#
# DESTINATION_DIR is the directory that the library is built into.
# This is not the same place where the object files are created.
#
DESTINATION_DIR=../lib

MAKE_FILE=-f otxml.mak
BUILD_TOOLS=../buildtools/
include ${BUILD_TOOLS}makeinclude


