//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:40 $
//
//==============================================================================

#include "AttributeSetAdapter.h"
#include "ot/xml/Attribute.h"

OT_SAX_NAMESPACE_BEGIN

using xml::Attribute;

const String sNull;

AttributeSetAdapter::AttributeSetAdapter(const AttributeSet& theSet) :
	m_theSet(theSet)
{
}


//==============================================================================
// AttributeSetAdapter::getIndex
//
// Look up the index of an attribute by XML 1.0 qualified name. 
//==============================================================================
int AttributeSetAdapter::getIndex(const String& qName) const
{
	RefPtr<Attribute> rpAddr = m_theSet.getAttribute(qName);
	if(rpAddr)
		return rpAddr->getIndex();
	else
		return -1;
}


//==============================================================================
// AttributeSetAdapter::getIndex
//
// Look up the index of an attribute by Namespace name. 
//==============================================================================
int AttributeSetAdapter::getIndex(const String& uri, const String& localPart) const
{
	RefPtr<Attribute> rpAddr = m_theSet.getAttribute(uri, localPart);
	if(rpAddr)
		return rpAddr->getIndex();
	else
		return -1;
}


//==============================================================================
// AttributeSetAdapter::getLength
//
// Return the number of AttributesImpl in the list. 
//==============================================================================
size_t AttributeSetAdapter::getLength() const
{
	return m_theSet.size();
}


//==============================================================================
// AttributeSetAdapter::getLocalName
//
// Look up an attribute's local name by index. 
//==============================================================================
String AttributeSetAdapter::getLocalName(size_t index) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(index);
	if(rpAttr)
	{
		return rpAttr->getName().getLocalName();
	}
	else
	{
		return sNull;
	}
}

//==============================================================================
// AttributeSetAdapter::getQName
//
// Look up an attribute's XML 1.0 qualified name by index. 
//==============================================================================
String AttributeSetAdapter::getQName(size_t index) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(index);
	if(rpAttr)
	{
		return rpAttr->getName().getRawName();
	}
	else
	{
		return sNull;
	}
}

//==============================================================================
// AttributeSetAdapter::getType
//
// Look up an attribute's type by index. 
//==============================================================================
String AttributeSetAdapter::getType(size_t index) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(index);
	if(rpAttr)
	{
		return rpAttr->getType();
	}
	else
	{
		return sNull;
	}
}
  

//==============================================================================
// AttributeSetAdapter::getType
//
// Look up an attribute's type by XML 1.0 qualified name. 
//==============================================================================
String AttributeSetAdapter::getType(const String& qName) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(qName);
	if(rpAttr)
	{
		return rpAttr->getType();
	}
	else
	{
		return sNull;
	}
}


//==============================================================================
// AttributeSetAdapter::getType
//
// Look up an attribute's type by Namespace name. 
//==============================================================================
String AttributeSetAdapter::getType(const String& uri, const String& localName) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(uri, localName);
	if(rpAttr)
	{
		return rpAttr->getType();
	}
	else
	{
		return sNull;
	}
}


//==============================================================================
// AttributeSetAdapter::getURI
//
// Look up an attribute's Namespace URI by index. 
//============================================================================== 
String AttributeSetAdapter::getURI(size_t index) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(index);
	if(rpAttr)
	{
		return rpAttr->getName().getNamespaceURI();
	}
	else
	{
		return sNull;
	}
}


//==============================================================================
// AttributeSetAdapter::getValue
//
// Look up an attribute's value by index. 
//==============================================================================
String AttributeSetAdapter::getValue(size_t index) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(index);
	if(rpAttr)
	{
		return rpAttr->getValue();
	}
	else
	{
		return sNull;
	}
}


//==============================================================================
// AttributeSetAdapter::getValue
//
// Look up an attribute's value by XML 1.0 qualified name. 
//==============================================================================
String AttributeSetAdapter::getValue(const String& qName) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(qName);
	if(rpAttr)
	{
		return rpAttr->getValue();
	}
	else
	{
		return sNull;
	}
}

//==============================================================================
// AttributeSetAdapter::getValue
//
// Look up an attribute's value by Namespace name. 
//==============================================================================
String AttributeSetAdapter::getValue(const String& uri, const String& localName) const
{
	RefPtr<Attribute> rpAttr = m_theSet.getAttribute(uri, localName);
	if(rpAttr)
	{
		return rpAttr->getValue();
	}
	else
	{
		return sNull;
	}
}

OT_SAX_NAMESPACE_END
