//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:44 $
//
//==============================================================================

#include "LocatorAdaptor.h"

#include "ot/base/NullPointerException.h"
#include "ot/xml/Parser.h"
#include "ot/xml/Position.h"

OT_SAX_NAMESPACE_BEGIN

LocatorAdaptor::LocatorAdaptor(Parser* pParser, bool bResolveSystemIds) :
	m_rpParser(pParser),
	m_bResolveSystemIds(bResolveSystemIds)
{
	if(!pParser) throw NullPointerException();
}

//==============================================================================
// LocatorAdaptor::getColumnNumber
//
// Implementation of Locator::getColumnNumber()
//==============================================================================
long LocatorAdaptor::getColumnNumber() const
{
	const long col = m_rpParser->getCurrentPosition().getColumnNumber();
	return (col) ? col : -1;
}

//==============================================================================
// LocatorAdaptor::getLineNumber
//
// Implementation of Locator::getLineNumber()
//==============================================================================
long LocatorAdaptor::getLineNumber() const
{
	const long line = m_rpParser->getCurrentPosition().getLineNumber();
	return (line) ? line : -1;
}

//==============================================================================
// LocatorAdaptor::getPublicId
//
// Implementation of Locator::getPublicId()
//==============================================================================
String LocatorAdaptor::getPublicId() const
{
	return m_rpParser->getCurrentPosition().getPublicId();
}

//==============================================================================
// LocatorAdaptor::getSystemId
//
// Implementation of Locator::getSystemId()
//==============================================================================
String LocatorAdaptor::getSystemId() const
{
	return m_bResolveSystemIds
		? m_rpParser->getCurrentPosition().getResolvedSystemId()
		: m_rpParser->getCurrentPosition().getSystemId();
}

void LocatorAdaptor::setResolveSystemIds(bool bSet)
{
	m_bResolveSystemIds = bSet;
}

bool LocatorAdaptor::getResolveSystemIds() const
{
	return m_bResolveSystemIds;
}


OT_SAX_NAMESPACE_END
