//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:50 $
//
//==============================================================================

#include "MessageFormatter.h"

#include "ot/base/NumUtils.h"

#include <vector>
#include <stdarg.h>

OT_UTIL_NAMESPACE_BEGIN

static const String sUnknown = OT_T("?");

String MessageFormatter::Format(const String& spec, const String& arg1)
{
	return FormatImpl(spec, 1, &arg1);
}

String MessageFormatter::Format(const String& spec, const String& arg1,
									  const String& arg2)
{
	return FormatImpl(spec, 2, &arg1, &arg2);
}

String MessageFormatter::Format(const String& spec, const String& arg1,
									  const String& arg2,
									  const String& arg3)
{
	return FormatImpl(spec, 3, &arg1, &arg2, &arg3);
}

String MessageFormatter::Format(const String& spec, const String& arg1,
									  const String& arg2,
									  const String& arg3,
									  const String& arg4)
{
	return FormatImpl(spec, 4, &arg1, &arg2, &arg3, &arg4);
}

String MessageFormatter::Format(const String& spec, const String& arg1,
									  const String& arg2,
									  const String& arg3,
									  const String& arg4,
									  const String& arg5)
{
	return FormatImpl(spec, 5, &arg1, &arg2, &arg3, &arg4, &arg5);
}

String MessageFormatter::Format(const String& spec, const String& arg1,
									  const String& arg2,
									  const String& arg3,
									  const String& arg4,
									  const String& arg5,
									  const String& arg6)
{
	return FormatImpl(spec, 6, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6);
}

//==============================================================================
// MessageFormatter::FormatImpl
//
// Do the actual work of parsing the specification string and replacing all
// occurences of {n} with the appropriate parameter value.
//
//==============================================================================
String MessageFormatter::FormatImpl(const String& spec, size_t count, ...)
{
	va_list marker;
	va_start(marker, count);     /* Initialize variable arguments. */

	std::vector<const String*> argVec;

	for(size_t i=0; i<count; ++i)
	{
		argVec.push_back(va_arg(marker, const String*));
	}
	
	va_end(marker);              /* Reset variable arguments.      */

	size_t lastpos=0;
	String ret;
	while(true)
	{
		size_t beginPos=spec.find('{', lastpos);
		if(beginPos == String::npos)
		{
			ret += spec.substr(lastpos);
			break;
		}
		else
		{
			size_t endPos=spec.find('}', beginPos);
			if(endPos == String::npos)
			{
				ret += spec.substr(lastpos);
				break;
			}
			else
			{
				ret += spec.substr(lastpos, beginPos-lastpos);
				lastpos = endPos+1;
				String argNum = spec.substr(beginPos+1, endPos-beginPos);
				size_t nArg = NumUtils::ToInt(argNum);
				if(nArg < argVec.size())
				{
					ret += *(argVec[nArg]);
				}
				else
				{
					ret += sUnknown;
				}
			}
		}
	}

	return ret;
}

OT_UTIL_NAMESPACE_END
