//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:51 $
//
//==============================================================================

#include "Attribute.h"

OT_XML_NAMESPACE_BEGIN


Attribute::Attribute() :
	m_nIndex(-1)
{
}

Attribute::Attribute(const Attribute& rhs) :
	m_name(rhs.m_name),
	m_value(rhs.m_value),
	m_type(rhs.m_type),
	m_nIndex(rhs.m_nIndex)
{
}

Attribute::Attribute(const QName& name,
                     const String& type,
					 int nIndex) :
	m_name(name),
	m_type(type),
	m_nIndex(nIndex)
{
}

Attribute::Attribute(const QName& name,
                     const String& value,
                     const String&  type) :
	m_name(name),
	m_value(value),
	m_type(type),
	m_nIndex(-1)
{
}

//==============================================================================
// Attribute::isDefaulted
//
// Not a good name for a function I know, but this returns whether
// or not the attribute exists because it was specified in the XML
// stream or because it was generated by the DTD/Schema
//==============================================================================
bool Attribute::isDefaulted() const
{
	return (m_nIndex == -1);
}

//==============================================================================
// Attribute::getType
//
// Return the type of this attribute
//==============================================================================
const String& Attribute::getType() const
{
	return m_type;
}

//==============================================================================
// Attribute::getName
//
// Returns the attribute's name
//==============================================================================
const QName& Attribute::getName() const
{
	return m_name;
}

//==============================================================================
// Attribute::getValue
//
// Returns the attribute's value
//==============================================================================
const String& Attribute::getValue() const
{
	return m_value;
}

//==============================================================================
// Attribute::getIndex
//
// Returns the relative position that this attribute held within the
// attribute list within the source document
//==============================================================================
int Attribute::getIndex() const
{
	return m_nIndex;
}

//==============================================================================
// Attribute::setName
//
//==============================================================================
void Attribute::setName(const QName& name)
{
	m_name = name;
}

//==============================================================================
// Attribute::setValue
//
//==============================================================================
void Attribute::setValue(const String& value)
{
	m_value = value;
}

//==============================================================================
// Attribute::setType
//
//==============================================================================
void Attribute::setType(const String& type)
{
	m_type = type;
}

OT_XML_NAMESPACE_END

