//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.2 $
// $Date: 2003/09/05 22:21:33 $
//
//==============================================================================

#include "AttributeSetImpl.h"

#include "ot/base/debug.h"
#include "ot/base/NullPointerException.h"

OT_XML_NAMESPACE_BEGIN

RefPtr<Attribute> AttributeSetImpl::getAttribute(const String& uri, const String& localName) const
{
	for(AttributeVector::const_iterator iter = m_attributes.begin();
	    iter != m_attributes.end(); ++iter)
	{
		const QName& name = (*iter)->getName();
		if(uri == name.getNamespaceURI() && localName == name.getLocalName())
		{
			return (*iter);
		}
	}
	return 0;
}

RefPtr<Attribute> AttributeSetImpl::getAttribute(const String& rawName) const
{
	for(AttributeVector::const_iterator iter = m_attributes.begin();
	    iter != m_attributes.end(); ++iter)
	{
		const QName& name = (*iter)->getName();
		if(rawName == name.getRawName())
		{
			return (*iter);
		}
	}
	return 0;
}

RefPtr<Attribute> AttributeSetImpl::getAttribute(size_t index) const
{
	if(index >= m_attributes.size())
	{
		return 0;
	}
	else
	{
		return m_attributes[index];
	}
}

bool AttributeSetImpl::empty() const
{
	return (m_attributes.size() == 0);
}

size_t AttributeSetImpl::size() const
{
	return m_attributes.size();
}

bool AttributeSetImpl::addAttribute(Attribute* pAttribute)
{
	if(!pAttribute) throw NullPointerException();

	const String& qName = pAttribute->getName().getRawName();
	if(!getAttribute(qName))
	{
		m_attributes.push_back(pAttribute);
		return true;
	}
	else
	{
		return false;
	}
}

void AttributeSetImpl::removeAttribute(size_t index)
{
	if(index < m_attributes.size())
	{
		AttributeVector::iterator i = m_attributes.begin() + index;
		m_attributes.erase(i);
	}
}

void AttributeSetImpl::removeAll()
{
	m_attributes.clear();
}

OT_XML_NAMESPACE_END
