//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.2 $
// $Date: 2003/09/05 22:21:33 $
//
//==============================================================================

#include "Buffer.h"
#include "Entity.h"
#include "ot/base/debug.h"

OT_XML_NAMESPACE_BEGIN

//
// For atomic reads, the mimumum read buffer size must be able to 
// hold the longest possible character sequence which is 6 bytes
//
static const size_t MinReadSize = 6;

Buffer::Buffer(size_t size, const Entity& entity) : 
	m_pData(new CharType[size]),
	m_used(0),
	m_bEOF(false),
	m_bFull(false),
	m_size(size),
	m_entity(entity)
{
	OT_DBG_ASSERT(size > 0);
	OT_DBG_ASSERT(m_pData!=0);
}

Buffer::~Buffer()
{
	delete [] m_pData;
}

void Buffer::read()
{
	if(m_bEOF)
	{
		return;
	}

	if(m_bFull)
	{
		if(m_rpNext.isNull())
		{
			m_rpNext = new Buffer(m_size, m_entity);
			m_rpNext->read();
		}
	}
	else
	{
		const size_t available = m_size-m_used;
		OT_DBG_ASSERT(available >= MinReadSize);
		const RefPtr<Reader>& rpReader = m_entity.getReader();
		long count = rpReader->readAtomic(m_pData+m_used, available);
		
		if(count == Reader::EndOfFile)
		{
			m_bEOF = true;
		}
		else
		{
			OT_DBG_ASSERT(count <= (long)available && count > 0);
			m_used += count;
			m_bFull = (m_size-m_used) <= MinReadSize;
		}
	}
}

const Entity& Buffer::getEntity() const
{
	return m_entity;
}

OT_XML_NAMESPACE_END

