//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:53 $
//
//==============================================================================

#include "ContextString.h"
#include "ot/base/StringUtils.h"

#include <algorithm>

OT_XML_NAMESPACE_BEGIN

ContextString::ContextString() :
	m_startCol(0),
	m_lineNo(0)
{
}

ContextString::ContextString(const String& value, size_t startCol, size_t lineNo) :
	m_value(value),
	m_startCol(startCol),
	m_lineNo(lineNo)
{
}

size_t ContextString::getStartColumn() const
{
	return m_startCol;
}

size_t ContextString::getLine() const
{
	return m_lineNo;
}

const String& ContextString::getValue() const
{
	return m_value;
}

//==============================================================================
// ContextString::getNormalizedValue
//
// Normalize the string by replacing tab characters with spaces.
//==============================================================================
String ContextString::getNormalizedValue() const
{
	String ret = m_value;
	StringUtils::ReplaceAll(ret, '\t', OT_T(" "));
	return ret;
}

String ContextString::getPointerString(size_t pointerCol) const
{
	size_t offset = pointerCol - (m_startCol-1);
	if(offset > 1)
	{
		String strRet(offset-1, '-');
		strRet += OT_T("^");
		return strRet;
	}
	else
		return OT_T("^");
}

OT_XML_NAMESPACE_END

