//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:55 $
//
//==============================================================================

#include "Entity.h"

OT_XML_NAMESPACE_BEGIN

//==============================================================================
// Entity::Entity
//
//==============================================================================
Entity::Entity(EntityType type, const Entity* pParent, bool bExternallyDeclared) :
	DTDObject(bExternallyDeclared),
	m_type(type),
	m_pParent(pParent)
{
}

//==============================================================================
// Entity::Entity
//
//==============================================================================
Entity::Entity(EntityType type, const Entity* pParent, bool bExternallyDeclared, const String& name) : 
	DTDObject(bExternallyDeclared),
	m_type(type),
	m_name(name),
	m_pParent(pParent)
{
}

//==============================================================================
// Entity::~Entity
//
//==============================================================================
Entity::~Entity()
{
}

//==============================================================================
// Entity::getType
//
//==============================================================================
EntityType Entity::getType() const
{
	return m_type;
}

//==============================================================================
// Entity::getName
//
//==============================================================================
const String& Entity::getName() const
{
	return m_name;
}

bool Entity::operator==(const Entity& rhs) const
{
	return (this == &rhs); 
}

bool Entity::operator!=(const Entity& rhs) const
{
	return !(*this == rhs); 
}

//==============================================================================
// Entity::getResolvedSystemId
//
// Only ExternalEntities have SystemIDs, so delegate this to parent who will
// know what to do with it.
//==============================================================================
String Entity::getResolvedSystemId() const
{
	// guard against recursion
	OT_DBG_ASSERT(getParent() != 0);
	return getParent()->getResolvedSystemId();
}

//==============================================================================
// Entity::getBaseURI
//
// Only ExternalEntities have base URIs, so delegate this to parent who will
// know what to do with it.
//==============================================================================
String Entity::getBaseURI() const
{
	// guard against recursion
	OT_DBG_ASSERT(getParent() != 0);
	return getParent()->getBaseURI();
}

//==============================================================================
// Entity::getPublicId
//
// Only ExternalEntities have PublicIds, so delegate this to parent who will
// know what to do with it.
//==============================================================================
String Entity::getPublicId() const
{
	// guard against recursion
	OT_DBG_ASSERT(getParent() != 0);
	return getParent()->getPublicId();
}

//==============================================================================
// Entity::getSystemId
//
// Only ExternalEntities have SystemIds, so delegate this to parent who will
// know what to do with it.
//==============================================================================
String Entity::getSystemId() const
{
	// guard against recursion
	OT_DBG_ASSERT(getParent() != 0);
	return getParent()->getSystemId();
}

//==============================================================================
// Entity::getParent
//
//==============================================================================
const Entity* Entity::getParent() const
{
	return m_pParent;
}

//==============================================================================
// Entity::getTopParent
//
//==============================================================================
const Entity* Entity::getTopParent() const
{
	if(m_pParent)
		return m_pParent->getTopParent();
	else
		return this;
}

//==============================================================================
// Entity::validate
//
// Currently just a base implmentation that does nothing.
// See ExternalEntity if you want an active implementation.
//==============================================================================
void Entity::validate(ParserImpl& /*parser*/) const
{
}

OT_XML_NAMESPACE_END

