//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:25:57 $
//
//==============================================================================

#include "InternalEntity.h"
#include "Scanner.h"
#include "BufferRange.h"

#include "ot/base/debug.h"

OT_XML_NAMESPACE_BEGIN

InternalEntity::InternalEntity(EntityType type, const Entity* pParent, bool bExternallyDeclared, 
							   const String& name,
							   Reader* pReader, const StreamPosition& location,
							   bool bSimpleData) : 
	Entity(type, pParent, bExternallyDeclared, name),
	m_rpReader(pReader),
	m_bSimpleData(bSimpleData)
{
	m_startPosition = Scanner::GetPosition(*this, location);
}

InternalEntity::~InternalEntity()
{
}

//==============================================================================
// InternalEntity::getStartPosition
//
//==============================================================================
ScannerPosition InternalEntity::getStartPosition()
{
	return m_startPosition;
}

//==============================================================================
// InternalEntity::isParsed
//
//==============================================================================
bool InternalEntity::isParsed() const
{
	return true; // Internal entities are always parsed
}

//==============================================================================
// InternalEntity::isExternalEntity
//
//==============================================================================
bool InternalEntity::isExternalEntity() const
{
	return false; // Internal entities are never external!
}

//==============================================================================
// InternalEntity::isInternalEntity
//
//==============================================================================
bool InternalEntity::isInternalEntity() const
{
	return true;
}

//==============================================================================
// InternalEntity::load
//
//==============================================================================
void InternalEntity::load()
{
	// Internal entities are always loaded
}

//==============================================================================
// InternalEntity::getReader
//
//==============================================================================
RefPtr<Reader> InternalEntity::getReader() const
{
	return m_rpReader;
}

//==============================================================================
// InternalEntity::getBufferRange
//
// Return a BufferRange representing the entire entity.
//==============================================================================
BufferRange InternalEntity::getBufferRange() const
{
	OT_DBG_ASSERT(m_bSimpleData);
	return Scanner::GetEntityBuffer(m_startPosition);
}

//==============================================================================
// InternalEntity::isSimpleData
//
// Return a flag indicating that this InternalEntity contains only
// simple data, ie no markup or entity references.
//==============================================================================
bool InternalEntity::isSimpleData() const
{
	return m_bSimpleData;
}

OT_XML_NAMESPACE_END

