//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:26:05 $
//
//==============================================================================

#include "QName.h"

OT_XML_NAMESPACE_BEGIN

const String sEmpty;


bool QName::operator==(const QName& rhs) const
{
	return (m_rawName == rhs.m_rawName);
}

bool QName::operator!=(const QName& rhs) const
{
	return (m_rawName != rhs.m_rawName);
}

//==============================================================================
// QName::::operator<()
//
// Used for sorting QNames lexicographically.
// The XML Canonicalization recommendation suggests that attributes
// short be sorted, using the namespace uri as the primary key and the local
// name as the secondary key.
//==============================================================================
bool QName::operator<(const QName& rhs) const
{
	return (m_namespaceURI < rhs.m_namespaceURI ||
	       (m_namespaceURI == rhs.m_namespaceURI && getLocalName() < rhs.getLocalName()));
}

const String& QName::getRawName() const
{
	return m_rawName;
}

String QName::getPrefix() const
{
	if(m_delimPosition != String::npos)
	{
		return m_rawName.substr(0, m_delimPosition);
	}
	else
	{
		return sEmpty;
	}
}

String QName::getLocalName() const
{
	if(m_delimPosition != String::npos)
		return m_rawName.substr(m_delimPosition+1, (m_rawName.size()-(m_delimPosition+1)));
	else
		return m_rawName;
}

const String& QName::getNamespaceURI() const
{
	return m_namespaceURI;
}

size_t QName::getDelimPosition() const
{
	return m_delimPosition;
}

void QName::setNamespaceURI(const String& namespaceURI)
{
	m_namespaceURI = namespaceURI;
}

void QName::setRawName(const String& rawName)
{
	m_rawName = rawName;
	setDelimPosition();
}

//==============================================================================
// QName::getUniversalName
//
// This is an attempt at creating a "universal name" ie a name representing
// the namespace and localname as a single name.  No standard exists for how
// this should be represented so we adopt the convention used by John Cowan's
// Namespace XML Filter which uses a a caret (^) to seperate the namespace URI
// from the local name.
//
// To deal with special cases like xml:space, when the URI is empty we revert to
// using the prefix ":" localname notation.
//==============================================================================
String QName::getUniversalName() const
{
	if(!m_namespaceURI.empty())
		return m_namespaceURI + String(OT_T("^")) + getLocalName();
	else
		return m_rawName;
}

OT_XML_NAMESPACE_END

