//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:26:07 $
//
//==============================================================================

#include "ScannerPosition.h"
#include "Entity.h"

OT_XML_NAMESPACE_BEGIN

const String sNull;

ScannerPosition::ScannerPosition(Buffer* pBuffer) :
	m_pNext(NULL),
	m_rpBuffer(pBuffer),
	m_bufferOffset(0),
	m_streamPosition(),
	m_eofAction(softEOF)
{
}

ScannerPosition::ScannerPosition(Buffer* pBuffer,
                                 const StreamPosition& position) :
	m_pNext(NULL),
	m_rpBuffer(pBuffer),
	m_bufferOffset(0),
	m_streamPosition(position),
	m_eofAction(softEOF)
{
}

void ScannerPosition::setNextPosition(const ScannerPosition& next, EOFAction eofAction)
{
	OT_DBG_ASSERT(m_pNext == NULL);
	delete m_pNext;
	m_pNext = new ScannerPosition(next);
	m_eofAction = eofAction;
}

//==============================================================================
// ScannerPosition::getEntity
//
//==============================================================================
const Entity& ScannerPosition::getEntity() const
{
	return m_rpBuffer->getEntity();
}

//==============================================================================
// ScannerPosition::getColumnNumber
//
//==============================================================================
size_t ScannerPosition::getColumnNumber() const
{
	return m_streamPosition.getColNo();
}

//==============================================================================
// ScannerPosition::getLineNumber
//
//==============================================================================
size_t ScannerPosition::getLineNumber() const
{
	return m_streamPosition.getLineNo();
}

//==============================================================================
// ScannerPosition::getOffset
//
//==============================================================================
size_t ScannerPosition::getOffset() const
{
	return m_streamPosition.getOffset();
}

//==============================================================================
// ScannerPosition::getPublicId
//
//==============================================================================
String ScannerPosition::getPublicId() const
{
	if(m_rpBuffer)
	{
		return m_rpBuffer->getEntity().getPublicId();
	}
	else
	{
		return sNull;
	}
}

//==============================================================================
// ScannerPosition::getSystemId
//
//==============================================================================
String ScannerPosition::getSystemId() const
{
	if(m_rpBuffer)
	{
		return m_rpBuffer->getEntity().getSystemId();
	}
	else
	{
		return sNull;
	}
}

//==============================================================================
// ScannerPosition::getResolvedSystemId()
//
//==============================================================================
String ScannerPosition::getResolvedSystemId() const
{
	if(m_rpBuffer)
	{
		return m_rpBuffer->getEntity().getResolvedSystemId();
	}
	else
	{
		return String();
	}
}

//==============================================================================
// ScannerPosition::getStreamPosition
//
//==============================================================================
const StreamPosition& ScannerPosition::getStreamPosition() const
{
	return m_streamPosition;
}

//==============================================================================
// ScannerPosition::isValid
//
//==============================================================================
bool ScannerPosition::isValid() const
{
	return (!m_rpBuffer.isNull());
}

OT_XML_NAMESPACE_END
