//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:27:04 $
//
//==============================================================================

#include "XMLInputSource.h"

#include "ot/base/NullPointerException.h"

OT_XML_NAMESPACE_BEGIN

XMLInputSource::XMLInputSource()
{
}

XMLInputSource::XMLInputSource(InputStream* pInputStream) :
	m_rpInputStream(pInputStream)
{
	if(!pInputStream) throw NullPointerException();
}

XMLInputSource::XMLInputSource(Reader* pReader) :
	m_rpReader(pReader)
{
	if(!pReader) throw NullPointerException();
}

XMLInputSource::XMLInputSource(const String& systemId) :
	m_systemId(systemId)
{
}

RefPtr<InputStream> XMLInputSource::getInputStream() const
{
	return m_rpInputStream;
}

RefPtr<Reader> XMLInputSource::getReader() const
{
	return m_rpReader;
}

String XMLInputSource::getSystemId() const
{
	return m_systemId;
}

String XMLInputSource::getEncoding() const
{
	return m_encoding;
}

void XMLInputSource::setEncoding(const String& encoding)
{
	m_encoding = encoding;
}

String XMLInputSource::getPublicId() const
{
	return m_publicId;
}

void XMLInputSource::setSystemId(const String& systemId)
{
	m_systemId = systemId;
}

void XMLInputSource::setPublicId(const String& publicId)
{
	m_publicId = publicId;
}

void XMLInputSource::setReader(Reader* pReader)
{
	m_rpReader = pReader;
}

void XMLInputSource::setInputStream(InputStream* pInputStream)
{
	m_rpInputStream = pInputStream;
}

OT_XML_NAMESPACE_END
