//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:27:05 $
//
//==============================================================================

#include "CatalogDelegatorEntry.h"

OT_XMLCAT_NAMESPACE_BEGIN

const String sBlank;

CatalogDelegatorEntry::CatalogDelegatorEntry(const String& identifier, EntryType eEntryType,
                                             CatalogResolver::Preference ePreference,
                                             const URL& catalogUrl,
											 const CatalogResolver& resolver) :
	CatalogEntry(identifier, eEntryType, ePreference),
	m_catalogSet(resolver)
{
	m_catalogSet.addFile(catalogUrl);
}

//==============================================================================
// CatalogDelegatorEntry::getCatalogSet
//
//==============================================================================
const CatalogSet& CatalogDelegatorEntry::getCatalogSet() const
{
	return m_catalogSet;
}

//==============================================================================
// CatalogDelegatorEntry::resolveEntity
//
// When delagating a request to a new catalog, only the public Id or system Id
// that caused the match is used for further matching.
//
//==============================================================================
bool CatalogDelegatorEntry::resolveEntity(const String& publicId, const String& systemId,
	const CatalogFile::List& catList,
	String& returnUri, bool& bDelegated) const
{
	if(m_ePreference == CatalogResolver::SYSTEM && !systemId.empty())
	{
		return false;
	}

	if(m_eEntryType == PUBLIC && publicId.substr(0, m_identifier.size()) == m_identifier)
	{
		bDelegated = true;
		bool bDummy = false;
		return m_catalogSet.resolveEntity(publicId, sBlank, catList, returnUri, bDummy);
	}
	else if(m_eEntryType == SYSTEM && systemId.substr(0, m_identifier.size()) == m_identifier)
	{
		bDelegated = true;
		bool bDummy = false;
		return m_catalogSet.resolveEntity(sBlank, systemId, catList, returnUri, bDummy);
	}
	else
	{
		return false;
	}
}

//==============================================================================
// CatalogDelegatorEntry::resolveURI
//
//==============================================================================
bool CatalogDelegatorEntry::resolveURI(const String& uri, 
                                       const CatalogFile::List& catList,
                                       String& returnUri, bool& bDelegated) const
{
	if(m_eEntryType == URI && uri.substr(0, m_identifier.size()) == m_identifier)
	{
		bDelegated = true;
		bool bDummy = false;
		return m_catalogSet.resolveURI(uri, catList, returnUri, bDummy);
	}
	else
	{
		return false;
	}
}

OT_XMLCAT_NAMESPACE_END
