//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:27:06 $
//
//==============================================================================

#include "CatalogLeafEntry.h"

OT_XMLCAT_NAMESPACE_BEGIN

CatalogLeafEntry::CatalogLeafEntry(const String& identifier, EntryType eEntryType,
                                   CatalogResolver::Preference ePreference,
                                   const String& uriReference) :
	CatalogEntry(identifier, eEntryType, ePreference),
	m_uriReference(uriReference)
{
}

//==============================================================================
// CatalogLeafEntry::getUriReference
//
//==============================================================================
const String& CatalogLeafEntry::getUriReference() const
{
	return m_uriReference;
}

//==============================================================================
// CatalogLeafEntry::resolveEntity
//
//==============================================================================
bool CatalogLeafEntry::resolveEntity(const String& publicId, const String& systemId,
                                     const CatalogFile::List& /*catList*/,
                                     String& returnUri, bool& /*bDelegated*/) const
{
	if(m_eEntryType == SYSTEM && m_identifier == systemId)
	{
		returnUri = m_uriReference;
		return true;
	}
	else if(m_eEntryType == REWRITE_SYSTEM && 
			systemId.substr(0, m_identifier.length()) == m_identifier)
	{
		returnUri = m_uriReference + systemId.substr(m_identifier.length());
		return true;
	}
	else if(m_ePreference == CatalogResolver::SYSTEM && !systemId.empty())
	{
		return false;
	}
	else if(m_eEntryType == PUBLIC && m_identifier == publicId)
	{
		returnUri = m_uriReference;
		return true;
	}
	else
	{
		return false;
	}
}

//==============================================================================
// CatalogLeafEntry::resolveURI
//
//==============================================================================
bool CatalogLeafEntry::resolveURI(const String& uri, const CatalogFile::List& /*catList*/,
                                  String& returnUri, bool& /*bDelegated*/) const
{
	if(m_eEntryType == URI && m_identifier  == uri)
	{
		returnUri = m_uriReference;
		return true;
	}
	else if(m_eEntryType == REWRITE_URI && 
			uri.substr(0, m_identifier.length()) == m_identifier)
	{
		returnUri = m_uriReference + uri.substr(m_identifier.length());
		return true;
	}
	else
	{
		return false;
	}
}

OT_XMLCAT_NAMESPACE_END
