//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2002 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.1.1.1 $
// $Date: 2002/12/14 00:27:08 $
//
//==============================================================================

#include "SAXCountHandler.h"

#include "ot/io/Console.h"
#include "ot/io/File.h"
#include "ot/net/URL.h"
#include "ot/net/MalformedURLException.h"
#include "ot/sax/SAXParseException.h"

using namespace ot;
using namespace ot::io;
using namespace ot::net;

SAXCountHandler::SAXCountHandler() :
	m_charCount(0),
	m_elementCount(0),
	m_attrCount(0)
{
}

//==============================================================================
// SAXCountHandler::startDocument
//
//==============================================================================
void SAXCountHandler::startDocument()
{
	m_charCount = m_elementCount = m_attrCount = 0;
}

//==============================================================================
// SAXCountHandler::endDocument
//
//==============================================================================
void SAXCountHandler::endDocument()
{
	Console::Out()->print(OT_T("Characters: "));
	Console::Out()->print(m_charCount);
	Console::Out()->print(OT_T(" Elements: "));
	Console::Out()->print(m_elementCount);
	Console::Out()->print(OT_T(" Attributes: "));
	Console::Out()->println(m_attrCount);
}

//==============================================================================
// SAXCountHandler::characters
//
//==============================================================================
void SAXCountHandler::characters(const CharType* /*pChars*/, size_t length)
{
	m_charCount += length;
}

//==============================================================================
// SAXCountHandler::startElement
//
//==============================================================================
void SAXCountHandler::startElement(const String& /*namespaceURI*/,
                                  const String& /*localName*/,
                                  const String& /*qName*/,
                                  const Attributes& atts)
{
	m_elementCount++;
	m_attrCount += atts.getLength();
}

// ErrorHandler methods...
void SAXCountHandler::error(const SAXParseException& exception)
{
	reportError(OT_T("Error"), exception);
}

void SAXCountHandler::fatalError(const SAXParseException& exception)
{
	reportError(OT_T("Fatal"), exception);
}

void SAXCountHandler::warning(const SAXParseException& exception)
{
	reportError(OT_T("Warning"), exception);
}

void SAXCountHandler::reportError(const String& type, const SAXParseException& exception)
{
	const String& systemId = exception.getSystemId();
	String fileName = systemId;

	if(!systemId.empty())
	{
		try
		{
			URL url(systemId);
			File file(url.getFile());
			fileName = file.getName();
		}
		catch(MalformedURLException&)
		{
		}
	}

	Console::Err()->print(fileName);
	Console::Err()->print(OT_T(" ["));
	Console::Err()->print(exception.getLineNumber());
	Console::Err()->print(OT_T(":"));
	Console::Err()->print(exception.getColumnNumber());
	Console::Err()->print(OT_T("] : "));
	Console::Err()->println(type + String(OT_T(": ")) + exception.getMessage());
}
