//==============================================================================
// This file is part of the OpenTop C++ Library.
// (c) Copyright 2000-2003 ElCel Technology Limited
//
// This file may be distributed and/or modified under the terms of the
// GNU General Public License version 2 as published by the Free Software
// Foundation and appearing in the file LICENSE.GPL included in the
// packaging of this file.
//
// Licensees holding valid OpenTop Commercial licences may use this file
// in accordance with the OpenTop Commercial License Agreement included
// with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// See http://www.elcel.com/products/opentop/pricing.html or email
// sales@elcel.com for information about OpenTop Commercial License Agreements.
//
//==============================================================================
//
// $Revision: 1.2 $
// $Date: 2003/09/08 21:00:15 $
//
//==============================================================================
//
// tcpserver sample application
//
// Class Listener
// --------------
//
// An instance of this class is created to whenever a new TCP socket connection
// is established with a client.  
// 
// This class handles all communication with the client over the socket.  It
// will run either synchronously or concurrently depending on the threading
// model.  In single-threaded mode the run() method is executed within the
// context of the main thread, so control is not returned to the main program
// until the client connection terminates.
//
//==============================================================================

#include "Listener.h"
#include "ClientHandler.h"

#include "ot/base/Thread.h"
#include "ot/net/ServerSocket.h"
#include "ot/io/Console.h"
#include "ot/io/IOException.h"

using namespace ot::io;

#define COUT Console::cout()
#define CERR Console::cerr()

Listener::Listener(ServerSocket* pSocket) :
#ifdef OT_MT
	Thread(OT_T("Listener")),
#endif
	m_rpSocket(pSocket)
{
}

void Listener::stop()
{
	try
	{
		COUT << OT_T("closing listening socket") << endl;
		m_rpSocket->close();
		COUT << OT_T("closed listening socket") << endl;
#ifdef OT_MT
		interrupt();
#endif
	}
	catch(IOException& e)
	{
		COUT << e.toString() << endl;
	}
}

void Listener::run()
{
#ifdef OT_MT
	COUT << OT_T("The listening thread is: ") << Thread::CurrentThread()->getName() << endl;
#endif
	try
	{
		while(true)
		{
			RefPtr<Socket> rpSocket = m_rpSocket->accept();
#ifdef OT_MT
			RefPtr<Thread> rpClientThread = new Thread(new ClientHandler(rpSocket.get(), this));
			rpClientThread->start();
#else
			RefPtr<ClientHandler> rpClientHandler = new ClientHandler(rpSocket.get(), this);
			rpClientHandler->run();
#endif
		}
	}
	catch(Exception& e)
	{
		CERR << OT_T("accept() failed: ") << e.toString() << endl;
	}
	COUT << OT_T("Listener terminated") << endl;
}

